#!/bin/tcsh
#
# Usage:
# source bgdFlareGTI.csh EVTLIST [TEST]
#
# A background flare GTI will be defined based upon the data in the events file
# EVTLIST.  (Parameters OBSID, Instrument name and exposure ID no longer need be
# provided as these are read from the header of EVTLIST.)
#
# Background flare GTIs may be defined either from an existing light curve or
# by using the ESPFILT routine.  If the optional second parameter ("TEST") is a
# positive real number and not "ESPFILT" (sensitive to spelling but not
# capitalization), then this will be used as the high background threshold,
# provided that an existing light curve is found.  If no light curve is
# recognized, TEST specifies ESPFILT, or TEST is not provided, then ESPFILT will
# be used.  In general, ESPFILT is the better choice for automated processing
# unless a TEST threshold is somehow well-chosen and not simply arbitrary...

if (-e sas.log) then
    echo " " >>! sas.log
    echo "Now running bgdFlareGTI.csh with the following parameters:" >>! sas.log
    echo "    EVTlist = ${1}" >>! sas.log
    echo "    [TEST] = ${2}" >>! sas.log
endif

setenv EVTLIST_GTIgen ${1}
if ("${2}"x != x) then
    setenv TEST_GTIgen `echo ${2} | tr '[a-z]' '[A-Z]'`
else
    setenv TEST_GTIgen ESPFILT
endif

fkeypar ${1} OBS_ID
setenv OBSID_GTIgen `pget fkeypar value | tr -d "'" | tr -d '"'`

fkeypar ${1} INSTRUME
setenv INST_GTIgen `pget fkeypar value | tr -d "'" | tr -d '"' | sed -e "s/EPN/PN/g" -e "s/EMOS/M/g" | tr '[a-z]' '[A-Z]'`

fkeypar ${1} EXPIDSTR
setenv EXPID_GTIgen `pget fkeypar value | tr -d "'" | tr -d '"'`

echo " "
echo "bgdFlareGTI.csh for ${INST_GTIgen} exposure ${EXPID_GTIgen}:"

# Determine which method to use to define flaring background threshold
#
if (! (${TEST_GTIgen} == ESPFILT) ) then
    # If a background threshold value is given, check whether a light curve
    # exists.  If one does not, reset TEST to ESPFILT.
    #
    set FLARE_LClist = `/bin/ls -t P${OBSID_GTIgen}${INST_GTIgen}${EXPID_GTIgen}FBKTSR0000*.F* \
	|& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g"`
    if ("${FLARE_LClist}"x == x) then
	# if background flare light curve doesn't exist...
	#
	echo "*** No previous ${INST_GTIgen} background flare light curve has been found, ***"
	echo "    so ESPFILT will be used to define the background flare GTI."
	setenv TEST_GTIgen ESPFILT
	unset FLARE_LClist
    endif
endif

# Now to generate the GTI...
#
if (${TEST_GTIgen} == ESPFILT) then
    # If we are to use ESPFILT...
    #
    echo "   ${INST_GTIgen} background flare GTI for exposure ${EXPID_GTIgen} to be defined using ESPFILT"
    setenv OUT_FLARE_GTI P${OBSID_GTIgen}${INST_GTIgen}${EXPID_GTIgen}-gti.FIT
    if (! -e ${OUT_FLARE_GTI}) then
	espfilt eventset=${EVTLIST_GTIgen} method=corner -V 1 >>&! bgdFlareGTI.out
    else
	echo "   No new file generated because a GTI file already exists"
	echo "   To generate a new GTI file, first delete ${OUT_FLARE_GTI}"
    endif
else
    # If we are to use previously-generated light curve...
    echo "   ${INST_GTIgen} background flare GTI to be defined based upon pre-exising light curve"
    setenv FLARE_LC ${FLARE_LClist[1]}
    if ("${FLARE_LClist}" != $FLARE_LC) then
	echo "*** Warning: multiple ${INST_GTIgen} ${EXPID_GTIgen} background flare light curves found! ***"
	echo "    The files found are:"
        /bin/ls -l P${OBSID_GTIgen}${INST_GTIgen}${EXPID_GTIgen}FBKTSR0000*.F*
	echo "    Only the newest one (${FLARE_LC}) will be used."
    endif
    unset FLARE_LClist
    setenv OUT_FLARE_GTI `echo ${FLARE_LC} | sed -e 's/FBKTSR/FBKGTI/g' -e 's/FTZ/FIT/g'`
    switch (${TEST_GTIgen})
	case [0-9]*:
	    # If first character is a numeral, then assume input is a positive
	    # number (hence a valid threshold value)
	    #
	    echo "    using a background flare threshold value of ${TEST_GTIgen}."
	    breaksw
	default:
	    if (${INST_GTIgen} == PN) then
		setenv TEST_GTIgen 10.0
	    else
		setenv TEST_GTIgen 2.0
	    endif
	    echo "*** WARNING: the flare threshold value must be a postive number ***"
	    echo "    or set to the word ESPFILT."
	    echo "    It has been reset to the default PPS threshold for ${INST_GTIgen},"
	    echo "    ${TEST_GTIgen} counts/kilosecond/square arcmin"
	    breaksw
    endsw
    if (-e ${FLARE_LC}) then
	# At this point, it had better exist...
	tabgtigen table=${FLARE_LC} expression="RATE<${TEST_GTIgen}" gtiset=${OUT_FLARE_GTI} >>&! bgdFlareGTI.out
    endif
endif
unsetenv OBSID_GTIgen
unsetenv INST_GTIgen
unsetenv TEST_GTIgen
unsetenv EXPID_GTIgen
unsetenv EVTLIST_GTIgen
