#!/bin/tcsh
#
# This program generates both region filter strings to be used with EVSELECT
# and DS9 *.reg files.  It bases these region definitions upon PPS region *.ASC
# files as much as possible.  Output regions are written as *.reg files; any
# pre-existing *.reg files will be overwritten.
#
# Note that this script makes use of the perl script "calc".
#
# Usage:
# source buildRegFilt.csh IM_FILE SRC_RA SRC_DEC SRCNUM [SRC_IRAD [SRC_ORAD [BGD_IRAD [BGD_ORAD]]]]
#
# Key coordinate-conversion parameters, OBSID, instrument and the exposure ID
# are read from the header of IM_FILE.
#
# SRC_RA and SRC_DEC are the coordinates of the intended source (in degrees).
#
# The source number SRCNUM is used when writing region files.
#
# SRC_IRAD, SRC_ORAD, BGD_IRAD and BGD_ORAD are the radii used when omitting the
# core of the source PSF to mitigate pileup, when defining the source extraction
# region, and the inner and outer radii of the background extraction annulus.
# If any of these are not provided, the default values of 12", 45", 60" and 120"
# will be used.  (Note: the PPS default values for the latter three are 28", 60"
# and 180".)
#
# Using the OBSID value, this script first looks for the PPS region file that
# lists the equatorial coordinates of all field sources detected in the
# combined data from all three EPIC cameras (P${OBSID_reg}EPX000REGION0000.ASC).
# The radii around the sources in this file are sized according to how bright or
# extended the source is, and are frequently smaller than 60".
#
# If this file is not available, then source region filters corresponding to
# individual sources will be used, preferably using one from the same instrument.
# Unlike the *EPX000REGION* files, these define a fixed radius of 60" around
# every field source.  Also, the first few regions need to be skipped as these
# define PPS source and background regions for an individual source.

# Setup parameters
#
# Inputs:
#

if (-e sas.log) then
    echo " " >>! sas.log
    echo "Now running buildRegFilt.csh with the following parameters:" >>! sas.log
    echo "    Image = ${1}" >>! sas.log
    echo "    SRC_RA = ${2}" >>! sas.log
    echo "    SRC_DEC = ${3}" >>! sas.log
    echo "    SRCNUM = ${4}" >>! sas.log
    if (${?5}) echo "    [SRC_IRAD] = ${5}" >>! sas.log
    if (${?6}) echo "    [SRC_ORAD] = ${6}" >>! sas.log
    if (${?7}) echo "    [BGD_IRAD] = ${7}" >>! sas.log
    if (${?8}) echo "    [BGD_ORAD] = ${8}" >>! sas.log
endif    

setenv IMG_FILE_reg ${1}
setenv SRC_RA_reg ${2}
setenv SRC_DEC_reg ${3}
setenv SRC_NUM_reg ${4}
if ("${5}"x != x) then
    setenv SRC_IRAD_reg ${5}
else
    setenv SRC_IRAD_reg 12
endif
if ("${6}"x != x) then
    setenv SRC_ORAD_reg ${6}
else
    setenv SRC_ORAD_reg 45
endif
if ("${7}"x != x) then
    setenv BGD_IRAD_reg ${7} 
else 
    setenv BGD_IRAD_reg 60
endif
if ("${8}"x != x) then
    setenv BGD_ORAD_reg ${8} 
else 
    setenv BGD_ORAD_reg 120
endif

# Set this flag to 1 to use DS9 to convert coordinates of background sources
# from equatorial to physical; set this to 0 (or any value != 1) to use my own
# conversion based upon header keyword parameters.  The difference between these
# methods is generally much less than 1".  I trust DS9 more...
#
setenv useDS9_reg 1

# When defining source extraction regions, cutouts will be made around any
# background sources that might otherwise contaminate these regions.  However,
# an exception is made if the "background" object coincides with the desired
# source, because the list of "background" objects includes all objects detected
# within the field and we do not want to cut out the desired object.
#
# The radius to be used for this coincidence test may be either a fixed value
# (in arcsec) or may be based upon the size of the candidate background object.
# I recommend using a fixed radius, because if the background object is rather
# large it might formally overlap the source even if they are clearly distinct
# objects.  Note that the largest separation that may be accepted as a
# 2XMMi+DR7 positional match is 7"...
#
setenv coincidence_radius 10
# 
# This value must be an integer.  If it is <= 0, then the radius of the
# candidate background source is used.


# Get several ID parameters from the image header:
#
fkeypar "${IMG_FILE_reg}[0]" EXPIDSTR
setenv EXPID_reg `pget fkeypar value | tr -d "'" | tr -d '"'`

fkeypar "${IMG_FILE_reg}[0]" OBS_ID
setenv OBSID_reg `pget fkeypar value | tr -d "'" | tr -d '"'`

fkeypar "${IMG_FILE_reg}[0]" INSTRUME
setenv INST_reg `pget fkeypar value | tr -d "'" | tr -d '"' | sed -e "s/EPN/PN/g" -e "s/EMOS/M/g" | tr '[a-z]' '[A-Z]'`

echo "buildRegFilt.csh is now making region filters for ${INST_reg} exposure ${EXPID_reg}..."

# Establish region files to be used:
#
setenv RegFile_reg P${OBSID_reg}EPX000REGION0000.ASC
setenv SrcCirFileOut P${OBSID_reg}${INST_reg}${EXPID_reg}SR_CIR0${SRC_NUM_reg}.reg
setenv SrcAnnFileOut P${OBSID_reg}${INST_reg}${EXPID_reg}SR_ANN0${SRC_NUM_reg}.reg
setenv BgdAnnFileOut P${OBSID_reg}${INST_reg}${EXPID_reg}BG_ANN0${SRC_NUM_reg}.reg
setenv BgdObjFileOut P${OBSID_reg}${INST_reg}${EXPID_reg}BG_OBJ0${SRC_NUM_reg}.reg
setenv SRCREGfile 0
# echo "debug: RegFile_reg   initialized as ${RegFile_reg}"
# echo "debug: SrcCirFileOut initialized as ${SrcCirFileOut}"
# echo "debug: SrcAnnFileOut initialized as ${SrcAnnFileOut}"
# echo "debug: BgdAnnFileOut initialized as ${BgdAnnFileOut}"
# echo "debug: BgdObjFileOut initialized as ${BgdObjFileOut}"
#
if (! -e $RegFile_reg) then
    # If expected file is not found, use any region file of a similar type:
    #
    set RegFileList = `/bin/ls -t *EPX000REGION0000.ASC`
    if ("${RegFileList}"x != x) then
# echo "debug 1"
	setenv RegFile_reg ${RegFileList[1]}
# echo "debug 1a"
	if ("${RegFileList}" != "$RegFile_reg") then
	    echo " "
	    echo "buildRegFilt.csh:"
	    echo "*** Note: more than one region file of the form *EPX000REGION0000.ASC was found"
	    echo "    Only the newest one ($RegFile_reg) is being used."
	endif
    else
	# If no region files of the preferred type (global EPIC source list) are
	# found, then use instrument-specific source lists, prioritizing ones
	# for the same instrument.  It is inconsequential which source these
	# were originally for.
	#
	set RegFileList_A = `/bin/ls *${OBSID_reg}${INST_reg}*SRCREG*.ASC`
	set RegFileList_B = `/bin/ls *${OBSID_reg}*SRCREG*.ASC`
	set RegFileList = ( ${RegFileList_A} ${RegFileList_B} )
	unset RegFileList_A
	unset RegFileList_B
	if ("${RegFileList}"x != x) then
#echo "debug 2"
	    setenv RegFile_reg ${RegFileList[1]}
	    setenv SRCREGfile 1
	    setenv BgdObjFileOut `echo $RegFile_reg | sed -e 's/SRCREG/BG_OBJ/g' -e 's/ASC/reg/g'`
#echo "debug 2a"
#	    echo "debug: BgdObjFileOut redefined   as ${BgdObjFileOut}"
	else
	    echo " "
	    echo "buildRegFilt.csh:"
	    echo "Warning: no source coordinate lists were found."
	    echo "    It is therefore assumed that there are no other sources in the field"
#echo "debug 3"
	    setenv SRCREGfile -1
#echo "debug 3a"
	endif
    endif
    unset RegFileList
endif
#
# At this point, RegFile_reg holds the name of the chosen region file source
# list or is blank if no region files were found.
# SRCREGfile = 0 if the region file is of the preferred type (a global EPIC source list);
# SRCREGfile = 1 if it is an instrument- and source-specific *SRCREG* file;
# SRCREGfile = -1 if no region files were found.

#echo "debug 4"
setenv srcORadius  `calc 20.0 * ${SRC_ORAD_reg}`
setenv bgdIRadius  `calc 20.0 * ${BGD_IRAD_reg}`
setenv bgdORadius  `calc 20.0 * ${BGD_ORAD_reg}`
setenv srcIRadius  `calc 20.0 * ${SRC_IRAD_reg}`
setenv srcORadius  `calc 20.0 * ${SRC_ORAD_reg}`
#echo "debug 4a"

# Circular source extraction region:
#
if (-e $SrcCirFileOut) /bin/rm $SrcCirFileOut
echo "# Region file format: DS9 version 4.1" > $SrcCirFileOut
echo "# Filename: ${IMG_FILE_reg}" >> $SrcCirFileOut
echo 'global color=cyan dashlist=8 3 width=1 font="helvetica 10 normal" select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1' >> $SrcCirFileOut
echo "fk5" >> $SrcCirFileOut
echo circle\(${SRC_RA_reg},${SRC_DEC_reg},${SRC_ORAD_reg}\"\) \# width\=2 >> $SrcCirFileOut

#
# Get parameters needed for generating region filters:
#
if ( $useDS9_reg == 1 ) then
    ds9 $IMG_FILE_reg -log -regions $SrcCirFileOut -regions system physical -regions format ciao -regions save tempFile.reg -exit
    set SrcCoordsList = `cat tempFile.reg | grep circle | tr '[A-Za-z,()"#=]' ' '`
#echo "debug 5"
    setenv srcX_coord ${SrcCoordsList[1]}
    setenv srcY_coord ${SrcCoordsList[2]}
#echo "debug 5a"
    unset SrcCoordsList
    /bin/rm tempFile.reg
else
#echo "debug 6"
    fkeypar "${IMG_FILE_reg}[0]" REFXCRVL
    setenv refXvalue `pget fkeypar value`
    fkeypar "${IMG_FILE_reg}[0]" REFYCRVL
    setenv refYvalue `pget fkeypar value`
    fkeypar "${IMG_FILE_reg}[0]" REFXCDLT
    setenv refXdelta `pget fkeypar value`
    fkeypar "${IMG_FILE_reg}[0]" REFYCDLT
    setenv refYdelta `pget fkeypar value`
    fkeypar "${IMG_FILE_reg}[0]" REFXCRPX
    setenv refXpixel `pget fkeypar value`
    fkeypar "${IMG_FILE_reg}[0]" REFYCRPX
    setenv refYpixel `pget fkeypar value`
    setenv srcX_coord `calc $refXpixel - ( $refXvalue - $SRC_RA_reg ) / $refXdelta`
    setenv srcY_coord `calc $refYpixel - ( $refYvalue - ($SRC_DEC_reg) ) / $refYdelta`
#echo "debug 6a"
endif
#
#echo "debug 7"
setenv SrcCirRegStr "((X,Y) IN circle(${srcX_coord},${srcY_coord},${srcORadius}))"
#echo "debug 7a"

# Annular source extraction region (in case of pileup):
#
#echo "debug 8"
setenv SrcAnnRegStr "((X,Y) IN circle(${srcX_coord},${srcY_coord},${srcORadius})) && !((X,Y) IN circle(${srcX_coord},${srcY_coord},${srcIRadius}))"
#echo "debug 8a"
if (-e $SrcAnnFileOut) /bin/rm $SrcAnnFileOut
echo "# Region file format: DS9 version 4.1" > $SrcAnnFileOut
echo "# Filename: ${IMG_FILE_reg}" >> $SrcAnnFileOut
echo 'global color=cyan dashlist=8 3 width=1 font="helvetica 10 normal" select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1' >> $SrcAnnFileOut
echo "fk5" >> $SrcAnnFileOut
echo annulus\(${SRC_RA_reg},${SRC_DEC_reg},${SRC_IRAD_reg}\",${SRC_ORAD_reg}\"\) \# width\=2 >> $SrcAnnFileOut

# Background annulus extraction region:
#
#echo "debug 9"
setenv BgdAnnRegStr "((X,Y) IN circle(${srcX_coord},${srcY_coord},${bgdORadius})) && !((X,Y) IN circle(${srcX_coord},${srcY_coord},${bgdIRadius}))"
#echo "debug 9a"
if (-e $BgdAnnFileOut) /bin/rm $BgdAnnFileOut
echo "# Region file format: DS9 version 4.1" > $BgdAnnFileOut
echo "# Filename: ${IMG_FILE_reg}" >> $BgdAnnFileOut
echo 'global color=green dashlist=8 3 width=1 font="helvetica 10 normal" select=1 highlite=1 dash=0 fixed=0 edit=1 move=1 delete=1 include=1 source=1' >> $BgdAnnFileOut
echo "fk5" >> $BgdAnnFileOut
#echo "debug 10"
if (${?COMMONNAME}) then
#    echo "debug: COMMONNAME is set to $COMMONNAME"
    setenv COMMONNAME_reg "${COMMONNAME}"
#    setenv COMMONNAME_reg `echo ${COMMONNAME} | tr -d ' '`
else
    setenv COMMONNAME_reg Source
endif
#echo "debug 10a"
echo annulus\(${SRC_RA_reg},${SRC_DEC_reg},${BGD_IRAD_reg}\",${BGD_ORAD_reg}\"\) \# width\=2 font\=\"helvetica 18 normal\" text\=\{${COMMONNAME_reg} background region\} background >> $BgdAnnFileOut
unsetenv COMMONNAME_reg

# Region defining/excluding all field sources:
#
if ($SRCREGfile < 0) then
    # If no regions found, assume there are no other sources and set up harmless
    # dummy value for exclude areas:
    #
#echo "debug 11"
    setenv BgdObjRegStr "!((X,Y) IN circle(999999.5,999999.5,0.5))"
#echo "debug 11a"
    /bin/cp $SrcCirFileOut $BgdObjFileOut
    echo "    The all-other-source background filter has been set to a harmless value"
else
    # Otherwise, get RA,Dec,Radius triplets for all sources:
    #
    if (-e $BgdObjFileOut) /bin/rm $BgdObjFileOut
    cat $RegFile_reg | sed -e 's/green/red/g' -e 's/circle/-circle/g' -e 's/edit=0/edit=1/g' -e 's/move=0/move=1/g' > $BgdObjFileOut
    set BgdObjParamsDeg=`grep circ $RegFile_reg | sed 's/fk5;//g' | tr '[A-Za-z,()"#=]' ' '`
    #
    if ($useDS9_reg == 1) then
	# Using DS9 to convert from equatorial to physical coordinates:
	#
#echo "debug 12"
	setenv RegFileOut_reg `echo $RegFile_reg | sed 's/ASC/reg/g'`
#echo "debug 12a"
	ds9 $IMG_FILE_reg -log -regions $RegFile_reg -regions system physical -regions format ciao -regions save $RegFileOut_reg -exit
	set BgdObjParamsPhys = `grep circle $RegFileOut_reg | tr '[A-Za-z,()"#=]' ' '`
    endif
    #
    # Parse the first triplet and initialize BgdObjRegStr...
    #
    set BgdObjCtr = 1
    if ($SRCREGfile == 1) @ BgdObjCtr+=9  # skip first thee triplets if using a SRCREG file
    if ($useDS9_reg == 1) then
#echo "debug 13"
	setenv SRC_RA_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	setenv srcX_coord ${BgdObjParamsPhys[$BgdObjCtr]}
	@ BgdObjCtr++
	setenv SRC_DEC_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	setenv srcY_coord ${BgdObjParamsPhys[$BgdObjCtr]}
	@ BgdObjCtr++
	setenv SRC_ORAD_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	setenv srcORadius ${BgdObjParamsPhys[$BgdObjCtr]}
	@ BgdObjCtr++
#echo "debug 13a"
    else
#echo "debug 14"
	setenv SRC_RA_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	@ BgdObjCtr++
	setenv SRC_DEC_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	@ BgdObjCtr++
	setenv SRC_ORAD_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	@ BgdObjCtr++
	# Add an extra arcsec to radius to allow for errors in coordinate
	# conversion (which differ from DS9 typically no more than ~0.6")...
	setenv SRC_ORAD_reg `calc $SRC_ORAD_REG + 1`
	setenv srcX_coord `calc $refXpixel - ( $refXvalue - $SRC_RA_reg ) / $refXdelta`
	setenv srcY_coord `calc $refYpixel - ( $refYvalue - ($SRC_DEC_reg) ) / $refYdelta`
	setenv srcORadius  `calc 20.0 * $SRC_ORAD_reg`
#echo "debug 14a"
    endif
#echo "debug 15"
    setenv BgdObjRegStr "!((X,Y) IN circle(${srcX_coord},${srcY_coord},${srcORadius}))"
#echo "debug 15a"
    #
    # Measure distance between first background object and spectral source in arcsec,
    # then exclude from source regions if overlapping and not the same object:
    #
#echo "debug 16"
    setenv sep_asec `calc sqrt(((${3})-(${SRC_DEC_reg}))**2 + (cos(pi*((${3})+(${SRC_DEC_reg}))/2/180)*(${2}-${SRC_RA_reg}))**2)*3600`
#echo "debug 16a"
    #
    # Test if current background object overlaps area within inner background annulus:
#echo "debug 17"
    setenv test1 `calc nint(100*($SRC_ORAD_reg + $BGD_IRAD_reg - ${sep_asec}))`
#echo "debug 17a"
    # ...if test1 > 0 then some part of current background object overlaps...
    # [N.b. - "nint" rounds to an integer; csh "if" conditions do not work with floats]
    #
    # Test if the current background object and the desired spectral source coincide:
#echo "debug 18"
    if (${coincidence_radius} > 0) then
	setenv test2 `calc nint(100*(${sep_asec}-${coincidence_radius}))`
    else
	setenv test2 `calc nint(100*(${sep_asec}-${SRC_ORAD_reg}))`
    endif
#echo "debug 18a"
    # ...if test2 < 0 then current background object is assumed to be either the desired
    # source or a part of it and should not be cut out from the selection region
    # 
    if ((${test1} > 0) && (${test2} > 0)) then
	# If background object does not coincide with spectral target and has some
	# overlap with the region within the inner radius of the background annulus...
	#
	# Choose a radius to use when cutting out this bgd object from source regions...
	# ...preliminary value: half the PPS source radius:
#echo "debug 19"
	setenv cutoutRad `calc ${SRC_ORAD_reg} / 2`
	# ...don't let this be too small:
	setenv test1 `calc nint(100*(${cutoutRad}-10))` # <0 if preliminary radius < 10
	if ( ${test1} < 0 ) setenv cutoutRad 10
	# ...but do go below this threshold if full PPS radius is smaller:
	setenv test1 `calc nint(100*(${cutoutRad}-${SRC_ORAD_reg}))` # >0 if SRC_ORAD smaller
	if ( ${test1} > 0 ) setenv cutoutRad ${SRC_ORAD_reg}
	# ...finally, if this reaches more than halfway to the source, make it smaller:
	setenv test1 `calc nint(100*(${cutoutRad} - (${sep_asec} / 2)))`
	if ( ${test1} > 0 ) setenv cutoutRad `calc ${sep_asec} / 2`
	echo -circle\(${SRC_RA_reg},${SRC_DEC_reg},${cutoutRad}\"\) \# color=red >> $SrcCirFileOut
	echo -circle\(${SRC_RA_reg},${SRC_DEC_reg},${cutoutRad}\"\) \# color=red >> $SrcAnnFileOut
	setenv cutoutRad `calc ${cutoutRad} * 20`
	setenv ExtraRegStr " && !((X,Y) IN circle(${srcX_coord},${srcY_coord},${cutoutRad}))"
	setenv SrcCirRegStr "${SrcCirRegStr}${ExtraRegStr}"
	setenv SrcAnnRegStr "${SrcAnnRegStr}${ExtraRegStr}"
#echo "debug 19a"
	unsetenv ExtraRegStr
    endif
    #
    # Now append additional exclude positions to BgdObjRegStr...
    while ($BgdObjCtr < ${#BgdObjParamsDeg})
	if ($useDS9_reg == 1) then
#echo "debug 20"
	    setenv SRC_RA_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	    setenv srcX_coord ${BgdObjParamsPhys[$BgdObjCtr]}
	    @ BgdObjCtr++
	    setenv SRC_DEC_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	    setenv srcY_coord ${BgdObjParamsPhys[$BgdObjCtr]}
	    @ BgdObjCtr++
	    setenv SRC_ORAD_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	    setenv srcORadius ${BgdObjParamsPhys[$BgdObjCtr]}
	    @ BgdObjCtr++
#echo "debug 20a"
	else
#echo "debug 21"
	    setenv SRC_RA_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	    @ BgdObjCtr++
	    setenv SRC_DEC_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	    @ BgdObjCtr++
	    setenv SRC_ORAD_reg ${BgdObjParamsDeg[$BgdObjCtr]}
	    @ BgdObjCtr++
	    # Add an extra arcsec to radius to allow for errors in coordinate
	    # conversion (which are typically no more than ~0.6")...
	    setenv SRC_ORAD_reg `calc $SRC_ORAD_REG + 1`
	    setenv srcX_coord `calc $refXpixel - ( $refXvalue - $SRC_RA_reg ) / $refXdelta`
	    setenv srcY_coord `calc $refYpixel - ( $refYvalue - ($SRC_DEC_reg) ) / $refYdelta`
	    setenv srcORadius `calc 20.0 * $SRC_ORAD_reg`
#echo "debug 21a"
	endif
#echo "debug 22"
	setenv ExtraRegStr " && !((X,Y) IN circle(${srcX_coord},${srcY_coord},${srcORadius}))"
#	setenv BgdObjRegStr "${BgdObjRegStr}${ExtraRegStr}"
#echo "debug 22a"
	#
	# Measure distance between first background object and spectral source in arcsec,
	# then exclude from source regions if overlapping and not the same object:
	#
#echo "debug 23"
	setenv sep_asec `calc sqrt(((${3})-(${SRC_DEC_reg}))**2 + (cos(pi*((${3})+(${SRC_DEC_reg}))/2/180)*(${2}-${SRC_RA_reg}))**2)*3600`
#echo "debug 23a"
	#
	# Test if current background object overlaps area within inner background annulus:
	setenv test1 `calc nint(100*($SRC_ORAD_reg + $BGD_IRAD_reg - ${sep_asec}))`
	# ...if test1 > 0 then some part of current background object overlaps...
	#
	# Test if the current background object and the desired spectral source coincide:
#echo "debug 24"
	if (${coincidence_radius} > 0) then
	    setenv test2 `calc nint(100*(${sep_asec}-${coincidence_radius}))`
	else
	    setenv test2 `calc nint(100*(${sep_asec}-${SRC_ORAD_reg}))`
	endif
#echo "debug 24a"
	# ...if test2 < 0 then current background object is assumed to be either the desired
	# source or a part of it and should not be cut out from the selection region
	#
	# Test if current background object comes close to outer background annulus:
#echo "debug 25"
	setenv test3 `calc nint(100*(2*$SRC_ORAD_reg + 2*$BGD_ORAD_reg - ${sep_asec}))`
#echo "debug 25a"
	# ...if test3 > 0 then some part of current background object comes close...
	# 
#echo "debug 26"
	if (${test3} > 0) setenv BgdObjRegStr "${BgdObjRegStr}${ExtraRegStr}"
#echo "debug 26a"
	if ((${test1} > 0) && (${test2} > 0)) then
	    # If background object does not coincide with spectral target and has some
	    # overlap with the region within the inner radius of the background annulus...
	    #
	    # Choose a radius to use when cutting out this bgd object from source regions...
	    # ...preliminary value: half the PPS source radius:
#echo "debug 27"
	    setenv cutoutRad `calc ${SRC_ORAD_reg} / 2`
	    # ...don't let this be too small:
	    setenv test1 `calc nint(100*(${cutoutRad}-10))` # <0 if preliminary radius < 10
	    if ( ${test1} < 0 ) setenv cutoutRad 10
	    # ...but do go below this threshold if full PPS radius is smaller:
	    setenv test1 `calc nint(100*(${cutoutRad}-${SRC_ORAD_reg}))` # >0 if SRC_ORAD smaller
	    if ( ${test1} > 0 ) setenv cutoutRad ${SRC_ORAD_reg}
	    # ...finally, if this reaches more than halfway to the source, make it smaller:
	    setenv test1 `calc nint(100*(${cutoutRad} - (${sep_asec} / 2)))`
	    if ( ${test1} > 0 ) setenv cutoutRad `calc ${sep_asec} / 2`
	    echo -circle\(${SRC_RA_reg},${SRC_DEC_reg},${cutoutRad}\"\) \# color=red >> $SrcCirFileOut
	    echo -circle\(${SRC_RA_reg},${SRC_DEC_reg},${cutoutRad}\"\) \# color=red >> $SrcAnnFileOut
	    setenv cutoutRad `calc ${cutoutRad} * 20`
	    setenv ExtraRegStr " && !((X,Y) IN circle(${srcX_coord},${srcY_coord},${cutoutRad}))"
	    setenv SrcCirRegStr "${SrcCirRegStr}${ExtraRegStr}"
	    setenv SrcAnnRegStr "${SrcAnnRegStr}${ExtraRegStr}"
#echo "debug 27a"
	endif
	unsetenv ExtraRegStr
    end
    unsetenv test1
    unsetenv test2
    unset BgdObjCtr
endif
echo "# SASfilter: ${SrcCirRegStr}" >> $SrcCirFileOut
echo "# SASfilter: ${SrcAnnRegStr}" >> $SrcAnnFileOut
echo "# SASfilter: ${BgdAnnRegStr}" >> $BgdAnnFileOut
echo "# SASfilter: ${BgdObjRegStr}" >> $BgdObjFileOut

unset BgdObjParamsDeg
if ($useDS9_reg == 1) unset BgdObjParamsPhys
unsetenv cutoutRad
unsetenv sep_asec
unsetenv srcX_coord
unsetenv srcY_coord
unsetenv bgdIRadius
unsetenv bgdORadius
unsetenv srcORadius
unsetenv srcIRadius
unsetenv RegFileOut_reg
unsetenv RegFile_reg
unsetenv SRCREGfile
unsetenv IMG_FILE_reg
unsetenv OBSID_reg
unsetenv EXPID_reg
unsetenv SRC_RA_reg
unsetenv SRC_DEC_reg
unsetenv SRC_NUM_reg
unsetenv INST_reg
unsetenv BGD_IRAD_reg
unsetenv BGD_ORAD_reg
unsetenv SRC_IRAD_reg
unsetenv SRC_ORAD_reg
if ($useDS9_reg != 1) then
    unsetenv refXvalue
    unsetenv refYvalue
    unsetenv refXdelta
    unsetenv refYdelta
    unsetenv refXpixel
    unsetenv refYpixel
endif
unsetenv useDS9_reg


# echo "The region filters have been defined as follows:"
# echo "To select the spectral source use this:"
# echo "    $SrcCirRegStr"
# echo "To omit the core of a piled-up source use this:"
# echo "    $SrcAnnRegStr"
# echo "The basic annular background region around this source is:"
# echo "    $BgdAnnRegStr"
# echo "The set of all background objects to be excluded from backgrounds is:"
# echo "    $BgdObjRegStr"
# Combine these latter two criteria to exclude background sources from
# the source background region...
