#!/bin/tcsh
#
# Command-line tool for retrieving XMM data for a single observation.
# 
# Retrieves ODF and PPS data (optionally checking first whether the ODF and PPS
# folders are already present).
# 
# Usage:
# fetchXMMobs.csh OBSID [FLAG [BASEPATH]]
# 
# where OBSID is the 10-digit XMM OBSID value for the desired data set.
#
# The second parameter, if present, is used as a flag for forcing the data to be
# downloaded even if it appears to already be present:
#   FLAG = 1: force downloading of ODF data
#        = 2: force downloading of PPS data
#        = 3: force downloading of both ODF and PPS
# Any other FLAG setting applies the default behavior of only getting ODF and/or
# PPS data if the odf/ and/or pps/ folders are not already present.
# 
# The optional BASEPATH is the directory within which the data are to be
# installed.  When BASEPATH is not provided, data are installed in the CWD.
#
#############

echo " "
echo "fetchXMMobs.csh: "

if ( $?OBSID_XMM ) setenv OBSID_XMM_SAV ${OBSID_XMM}
setenv OBSID_XMM ${1}

if ("${2}"x != x) then
    switch (${2})
	case 1:
	    set ForceFetchFlag = 1
	    breaksw
	case 2:
	    set ForceFetchFlag = 2
	    breaksw
	case 3:
	    set ForceFetchFlag = 3
	    breaksw
	default:
	    set ForceFetchFlag = 0
	    breaksw
    endsw
else
    set ForceFetchFlag = 0
endif

# debug
#
# This setting is temporary, to force the script to re-download PPS data every time.
# I'm doing this because I'm re-running jobs that had been interrupted, which usually
# was because the downloads were incomplete (parallel processes would clobber each
# other downloads).
#
set ForceFetchFlag = 2
#
# debug

if ("${3}"x == x) then
    # ...if parameter not set, then do nothing...
else
    if ((`pwd` != ${3}) && (`pwd`/ != $DataPath_XMM)) then
	# ...if not already in desired location...
	pushd ${3}
	set fetchInNewDirectory
    endif
endif

setenv levelStr ""
setenv fetchODF 0
setenv fetchPPS 0
if ((${ForceFetchFlag} == 1) || (${ForceFetchFlag} == 3)) setenv fetchODF 1
if ((${ForceFetchFlag} == 2) || (${ForceFetchFlag} == 3)) setenv fetchPPS 1
if (! -e ${OBSID_XMM}/odf) setenv fetchODF 1
if (! -e ${OBSID_XMM}/pps) setenv fetchPPS 1

if ( ${fetchODF} > 0 || ${fetchPPS} > 0) then
    echo "   Attempting to retrieve data for OBSID ${OBSID_XMM}"
    if (! $?JDK_HOME) then
	setenv JDK_HOME /System/Library/Frameworks/JavaVM.framework/Versions/CurrentJDK/Home
	setenv JDK_HOME_SET
    endif
    if ($fetchODF == 0) setenv levelStr " level=PPS"
    if ($fetchPPS == 0) setenv levelStr " level=ODF"
    aioclient -S xsa.esac.esa.int -P 2002 -L \"GET obsno=${OBSID_XMM}${levelStr}\"
    if ($?JDK_HOME_SET) then
	unsetenv JDK_HOME
	unsetenv JDK_HOME_SET
    endif
    set fileTest = `/bin/ls AIOSOCK*.tar |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g"`
    if ("${fileTest}"x != x) then
	tar xfm AIOSOCK*.tar
	/bin/rm AIOSOCK*.tar
    else
	if ($fetchPPS > 0) echo "*** ERROR!  No PPS products downloaded for ${OBSID_XMM}! ***"
    endif
    unset fileTest
    if ($fetchPPS == 0) then
	# if only obtaining ODF data, make sure tar ball is nested within OBSID_XMM/odf
	if (! -e ./${OBSID_XMM}) mkdir ${OBSID_XMM}
	if (! -e ./${OBSID_XMM}/odf) mkdir ${OBSID_XMM}/odf
	if (-e ${OBSID_XMM}.tar.gz) then
	    mv ${OBSID_XMM}.tar.gz ${OBSID_XMM}/odf/
	else
	    echo "*** ERROR!  No ODF data products downloaded for ${OBSID_XMM}! ***"
	endif
    endif
    if ($fetchODF == 1) then
	# if ODF files amongst new data, expand and remove tarballs
	pushd ${OBSID_XMM}/odf/
	if (-e ${OBSID_XMM}.tar.gz) then
	    tar xfmz ${OBSID_XMM}.tar.gz
	    /bin/rm -f ${OBSID_XMM}.tar.gz
	    tar xfm *${OBSID_XMM}.TAR
	    /bin/rm -f *${OBSID_XMM}.TAR
	else
	    echo "*** ERROR!  ODF data tarball not found in ${OBSID_XMM}/odf/ ! ***"
	endif
	popd
    endif
else
    echo "   No attempt made to download data."
    echo "   ${OBSID_XMM}/odf and ${OBSID_XMM}/pps already exist in `pwd`."
endif
unsetenv fetchODF
unsetenv fetchPPS
unsetenv levelStr
unset ForceFetchFlag

if (${?fetchInNewDirectory}) then
    # ...if CWD has been changed, change back again...
    popd
    unset fetchInNewDirectory
endif

if ($?OBSID_XMM_SAV) then
    # ...if there had been a previous OBSID_XMM setting, restore it...
    setenv OBSID_XMM $OBSID_XMM_SAV
    unsetenv OBSID_XMM_SAV
endif
