#!/bin/tcsh
#

#  This file should not require updating when the SAS software is
#  updated.  It is only necessary to ensure that ${SAS_DIR} is defined
#  to point to the correct directory before this script is invoked.
#
#  To set up earlier versions of SAS (if they are still installed),
#  simply redefine SAS_DIR before calling this script...
#
#  Modification for XMM automated processing: if a second parameter is an
#  existing file, the verbose output from cifbuild and odfingest will be
#  appended to this file.

if ($?XMMSASSET) then
   echo " "
   echo "        The SAS package appears to already be set up in this window."
   echo "        No changes have been made to the pre-existing configuration."
   echo " "
   # echo "If you are trying to re-configure SAS, try either this command:"
   # echo "          unsetenv XMMSASSET"
   # echo "followed by 'setsas', or simply try 'setsas' in a new window."
   # echo " "
else
   #  Before setting up the SAS tools, it might be useful to make sure
   #  FTOOLS are available:
   #
   if (! $?LHEASOFTSET) then
      echo " "
      echo "First, we are setting up HEASOFT so that xspec and ftools will be avaialble..."
      source /Users/Shared/Scripts/setlheasoft.csh
   endif
   #
   #  If SAS_DIR is defined before calling this script, then continue
   #  using this setting (allows users to work with earlier
   #  installations of SAS).  Otherwise, set this to the most recent
   #  version:
   #
   if (! $?SAS_DIR) setenv SAS_DIR /soft/sas/xmmsas_current
   #
   #  The SAS setup script ${SAS_DIR}/setsas.csh
   #  (a script produced by the SAS make file when SAS was installed)
   #  defines SAS_DIR and SAS_PATH so it is not formally necessary to
   #  define either of these here.  However, I set SAS_DIR in advance
   #  as a mechanism for selecting earlier SAS installations.
   #
   #  setenv SAS_DIR /soft/sas/xmmsas_current
   #  setenv SAS_PATH ${SAS_DIR}
   #
   #
   #  The next two variables are definitely helpful:
   #
   setenv SAS_CCFPATH /soft/sas/ccf
   setenv SAS_BROWSER open
   #
   #
   #  These others are not necessary but may be uncommented and tweaked: 
   #
   # setenv SAS_SUPPRESS_WARNING 1
   # setenv SAS_VERBOSITY 3
   #
   #  Note that verbosity can take values from 0 to 10; the higher the
   #  number, the more locquacious.  1 is default; anything higher than
   #  4 is primarily intended for debugging.
   #
   #
   #  These variables should not be set here, as their values will be
   #  specific to a particular data set:
   #
   # setenv SAS_CCF
   # setenv SAS_ODF
   # setenv SAS_ODFPATH
   #
   #
   #  Before calling the SAS-provided setup scripts, make sure
   #  DYLD_LIBRARY_PATH is defined, else these scripts will fail with
   #  an error (at the time of this writing (Jan 2007), this variable
   #  was defined by the LHEASOFT setup scripts):
   #
   if (! $?DYLD_LIBRARY_PATH ) setenv DYLD_LIBRARY_PATH
   #  
   #  
   #  Finally, call the SAS-provided script (which in turns calls yet
   #  another script):
   #  
   source ${SAS_DIR}/setsas.csh
   #
#  echo "   (n.b. - there is no need to re-define SAS_CCFPATH)"
#  echo " "
   #
   #  I have not made any modifications to the SAS-provided scripts.
   #  (I could have put some of the commands from within this script
   #  there, but I didn't want to have to recreate my customizations
   #  any time SAS gets updated.)
   #
   #
   #  The generic SAS setup script prepends ${SAS_DIR}/libextra to
   #  DYLD_LIBRARY_PATH .  At least some of the dynamic libraries
   #  provided therein are redundant with ones provided by fink in
   #  /sw/lib and may be older/somewhat stale.  Thus, prepend /sw/lib
   #  so that the Fink versions will be found first:
   #
   #  setenv DYLD_LIBRARY_PATH /sw/lib:${DYLD_LIBRARY_PATH}
   #
   #  Above line commented out 28 Oct 2008.  It seems that having
   #  /sw/lib *anywhere* in DYLD_LIBRARY_PATH causes rsync to attempt
   #  to use /sw/lib/libiconv.2.dylib , which in turn causes my
   #  "updatesasccf" function to fail.
   #
   #  Last of all, set a flag to indicate that SAS has been set up.
   #
   setenv XMMSASSET
endif
#
#
#  Before finishing setup, check when the CCF files were last updated
#  and echo a reminder to update these if necessary...
#
echo "^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^"
if (-e /soft/sas/ccf_timestamp) then
   cat /soft/sas/ccf_timestamp
   echo " "
   echo "If this was a while ago, update the CCF files using the alias  'updatesasccf'"
   # echo "(latest update list at xmm.vilspa.esa.es/external/xmm_sw_cal/calib/rel_notes/ )"
else
   echo "The timestamp of the last SAS CCF update couldn't be found"
   echo "   Make sure the CCF files are installed and are current"
   echo "   (Hint: consider the alias 'updatesasccf'...)"
endif
echo " "
#
#
#  If a path to the ODF directory is provided, use it to finish the
#  setup of the SAS environment; if it is not, suggest that the user
#  re-run this script after moving to their work directory.
#
echo "^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^     ^^^^^"
if ( "${1}x" != "x" ) then
   #  (If an argument - the path to the ODF files - has been provided...)
   #
   #  ccf.cif section
   #
   if ( -e ${1} ) then
      setenv SAS_ODF ${1}
      if ( -e ccf.cif ) then
         echo "   A file named 'ccf.cif' already exists and will be used as SAS_CCF."
         echo "   If you want to replace it, you must issue the command  'cifbuild'."
         # echo "   ***ccf.cif already exists!***"
         # echo "   Original moved to ccf.cif-save and new version being generated."
         # echo " "
         # mv ccf.cif ccf.cif-save
         set prevSASsetup
      else
         echo "Running 'cifbuild' to identify the relevant CCF files..."
	 if (-e tmp.out) /bin/rm -f tmp.out
         cifbuild >>&! tmp.out
	 if ( -e ${2} ) then
	    echo "Output from cifbuild:" >>! ${2}
	    cat tmp.out >>! ${2}
	 endif
	 echo "   CIFBUILD warnings and errors:"
	 grep -i " warning " tmp.out
	 grep -i " error " tmp.out
	 if (-e tmp.out) /bin/rm -f tmp.out
      endif
      setenv SAS_CCF ccf.cif
      #
      #
      #  *SUM.SAS section
      #
      echo " "
      if ( $?prevSASsetup ) then
         #  (If ccf.cif had already existed, test for & save prior *SUM.SAS file)
         set prevSUMSASfile = `/bin/ls -t *SUM.SAS |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g"`
         if ( "${prevSUMSASfile}"x != x ) then
            echo "   ${#prevSUMSASfile} previous *SUM.SAS file(s) already exist(s)."
	    echo "   The newest one is ${prevSUMSASfile[1]} and will be used as SAS_ODF."
            echo "   To replace it, use the command  'odfingest'  then redefine SAS_ODF."
	    setenv SAS_ODF ${prevSUMSASfile[1]}
            # echo "   *** A previous *SUM.SAS file already exists as well! ***"
            # echo "   Original moved to *SUM.SAS-save and new version being generated."
            # set prevsumfile=`/bin/ls -1 *SUM.SAS`
            # mv ${prevsumfile} "${prevsumfile}-save"
            # unset prevsumfile
            set runodfingest=0
         else
            # echo "   (if the previous line says '/bin/ls: No match.', just ignore it.)"
            set runodfingest=1
         endif
         unset prevSUMSASfile
         unset prevSASsetup
      else
         set runodfingest=1
      endif
      if ( ${runodfingest} == 1 ) then
         echo "Now running 'odfingest' to configure SAS to find your ODF data files..."
	 if (-e tmp.out) /bin/rm -f tmp.out
         odfingest >>&! tmp.out
	 if ( -e ${2} ) then
	    echo "Output from odfingest:" >>! ${2}
	    cat tmp.out >>! ${2}
	 endif
	 echo "   ODFINGEST warnings and errors:"
	 grep -i " warning " tmp.out
	 grep -i " error " tmp.out
	 echo " "
	 if (-e tmp.out) /bin/rm -f tmp.out
         unset runodfingest
      endif
      set sumfile=`/bin/ls -t *SUM.SAS`
      if ("${sumfile}"x != x) then
         setenv SAS_ODF ${sumfile[1]}
         echo "   Setting SAS_ODF to ${sumfile[1]}"
      else
         echo " ***ERROR***  No *SUM.SAS file found!  SAS_ODF cannot be set properly."
      endif
      unset sumfile
      echo " "
      echo "Your SAS setup is now complete.  Have fun."
   else
      # echo "It appears that an argument (${1}) has been passed to SETSAS."
      # echo "   However, this folder cannot be found so it will not be used as SAS_ODF."
      # echo "   Consequently, SAS_CCF is not set either."
      # echo " "
      echo "To complete the setup process, confirm that your CCFs are up-to-date,"
      echo "   cd to your work directory and issue the command  'setsas ODF_FOLDER',"
      echo "   where ODF_FOLDER is path to the folder containing the untarred ODF files."
   endif
else
   #  (If path to ODF files has not been provided...)
   echo "To complete the setup process, confirm that your CCFs are up-to-date,"
   echo "   cd to your work directory and issue the command  'setsas ODF_FOLDER',"
   echo "   where ODF_FOLDER is path to the folder containing the untarred ODF files."
endif
echo " "
