#!/bin/tcsh
#
# Usage:
# to provide values on the command line, these must be:
# xmmSpecPrep.csh DETID OBSID "IAUNAME" SRCNUM SDSS_RA SDSS_DEC "COMMONNAME"
# e.g.
# xmmSpecPrep.csh 014320 0305920101 "2XMM J010712.0+140844" 001 \
#            16.80014 14.14583 "1RXS J010712.9+140843"
#
# These parameters are:
# DETID = XMM detection ID.  Unique value for every detection of every object
#   in 2XMM catalogue.
# OBSID = XMM observation ID.
# IAUNAME = IAU name of 2XMM catalogue entry.  Enclosed in quotes if there are
#   any space characters.
# SRCNUM = 2XMM souce number (defined by 2XMM as an ordered list of objects
#   detected in a given OBSID, ranked by flux).
# SDSS_RA, SDSS_DEC = J2000.0 coordinates of SDSS spectroscopic target.
# COMMONNAME = alternative name of object; is a truely famous name (e.g., NGC
#   or Markarian numbers), if any are appropriate; else is a catalogue ID if
#   there are any non-optical and non-2MASS IDs; else "SDSS" if only IDs are
#   from SDSS, 2MASS, and/or XMM.  Must be enclosed in quotes if it includes
#   any space characters.
#
#####
#
# There are numerous parameters that are hard coded (i.e. - cannot be set from
# the command line) but can be modified by changing the definition at the start
# of this script.  These are below, following the parsing of the command line
# parameters.  To find these, search for the string "###".  However, there is
# one that is useful to define right from the start...
#
### Path to directory within which downloaded data are to be installed:
#
setenv DataPath_XMM "/Volumes/Archive/Data/XMM/"
#
# See below for additional hard-coded setting parameters.

# if (-e ${DataPath_XMM}${2}_${4}_summary.log) /bin/rm -f ${DataPath_XMM}${2}_${4}_summary.log

set SASstartTime = "`date`"

echo " "
echo "<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>"
echo "<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>"
echo " "
echo "Starting to process data for ${3}"
echo "                     (DETID ${1}; OBSID ${2}, SRCNUM ${4} ${7})"
echo "                     at ${SASstartTime}"
echo " "
echo "<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>"
echo "<><><><><><><><><><><><><><><><><><><><><><><><><><><><><><><>"
echo " "
# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (1) <><>"
# echo "<><>             obtaining and defining variables         <><>"
echo "<><><><> ...defining variables... <><><><>"
############
# (1) Obtain/define variables
#
# Parse input variables
#
if ( "${1}x" != "x" ) then
    setenv DETID_XMM	${1}
else
    echo "**** Warning: No XMM DETID value provided.  Using dummy value."
    setenv DETID_XMM	999999
endif
if ( "${2}x" != "x" ) then
    setenv OBSID_XMM	${2}
else
    echo "**** Warning: No XMM OBSID value provided.  Using dummy value."
    echo "     This will likely break many things because files will not be found."
    setenv OBSID_XMM	9999999999
endif
if ( "${3}x" != "x" ) then
    setenv IAUNAME_XMM	"${3}"
else
    echo "**** Warning: No XMM IAU Name provided.  Using dummy value."
    setenv IAUNAME_XMM	"2XMM J000000.0+000000"
endif
if ( "${4}x" != "x" ) then
    setenv SRCNUM_XMM	${4}
else
    echo "**** Warning: No XMM SRC_NUM value provided.  Using default value 001."
    setenv SRCNUM_XMM	001
endif
if ( "${5}x" != "x" ) then
    setenv SDSS_RA	${5}
else
    echo "**** Warning: No SDSS RA provided.  Using default value."
    setenv SDSS_RA	191.64687
endif
if ( "${6}x" != "x" ) then
    setenv SDSS_DEC	${6}
else
    echo "**** Warning: No SDSS Dec provided.  Using default value."
    setenv SDSS_DEC	2.36910
endif
if ( "${7}x" != "x" ) then
    setenv COMMONNAME	"${7}"
else
    echo "**** Warning: No common name provided.  Using default value GENERIC."
    setenv COMMONNAME	GENERIC
endif


# Count rates may be used to trigger pileup remediation strategies:
#
# setenv PN_8_RATE	0.3037
# setenv M1_8_RATE	0.0757
# setenv M2_8_RATE	0.0791
#
# Can we establish these directly from the data?
#
# Count rate thresholds depend upon windowing mode and (to a lesser extent) upon
# the filter.  These may be obtained from the header keywords SUBMODE and FILTER.

# Redshift will be needed in XSPEC analysis, but likely not before:
#
# setenv SDSS_REDSHIFT	0.07695

##### Other hard-coded parameters may be set here:
##### Other hard-coded parameters may be set here:
##### Other hard-coded parameters may be set here:

### Update XMM calibration files every time script is run?
#
setenv UpdateCCF NO
#
# If anything other than "YES", CCFs will not be updated with every run.

### Path to script files:
#
setenv ScriptPath_XMM "/Volumes/Archive/Data/XMM/scripts/"

# ### Path to directory within which downloaded data are to be installed:
# #
# setenv DataPath_XMM "/Volumes/Archive/Data/XMM/"
# 
# Now defined at start of script.  See above.

### Path to spectral analysis folders, to which spectral products will be copied:
#
setenv AnalysisPath "/Users/jonathan/Desktop/Projects/XMManalysis/"

### Name of output log file:
#
setenv SasOutLog sas.log

### Default SAS verbosity level:
#
setenv SAS_VERBOSITY_DEFAULT 1

### Do you want to force the script to attempt to extract events?
#
setenv ForceEvtExtract NO
#
# If YES, then epchain and emchain will be run in an effort to extract events
# unless there are already events files extracted by this pipeline.  If no,
# then PPS imaging events lists will be used whenever they are available and
# epchain/emchain will only be called when no events lists are found.
#
# Attempting to extract events has the advantage of finding alternative
# exposures that may have been missed by the PPS pipeline (I am unsure whether
# this ever happens), but at a cost of a lot of processing time.

### Should PPS pipeline spectra be used (when available)? 
#
setenv UsePPSspectra NO
#
# Any value other than "YES" (all caps) is taken as no.

### Force the re-extraction of spectra?
#
set reExtractSpec = YES
#
# If "YES", then spectra will be extracted anew, clobbering any previously-
# extracted spectra not generated by the pipeline; if anything other than "YES",
# then pre-existing spectra will be used whenever available.

### Use generic RMF files?
#
setenv UseCannedRMFs NO
#
# NOT YET IMPLEMENTED!
# If YES, then generic RMF files will be used.  Otherwise, use RMFs generated
# for this specific observation of this source, likely produced by this script.
# NOT YET IMPLEMENTED!

### Force the re-generation of RMF and ARF matrices?
#
set reGenerateMatrices = YES
#
# If "YES", then matrices will be generated anew, clobbering any previously-
# generated ones; if anything other than "YES", then pre-existing matrices will
# be used whenever available (including canned RMFs only if UseCannedRMFs==YES)

### Should PPS pipeline images be used (when available)? 
#
setenv UsePPSimages YES
#
# Any value other than "YES" (all caps) is taken as no.

### What method/count rate thresholds should be used to identify background flares?
#
setenv PNbgdFlareGTItest ESPFILT
setenv MOSbgdFlareGTItest ESPFILT
#
# These should either be set to "ESPFILT" or a positive real number.
# 
# When defining background flare GTIs, either of two tests may be used to define
# flares: a fixed (positive, real) count rate, or the routine ESPFILT.  The PPS
# does the former, applying count rate thresholds of 10.0 (PN) or 2.0 (MOS).
# ESPFILT defines a threshold based upon the distribution of count rates across
# the background light curve.  This has the advantage of being a non-arbitrary
# value based upon the data and the disadvantages of being more time-consuming
# and always rejecting the highest count rate intervals, even when these are not
# problematic flares.  I recommend using ESPFILT.

### Copy alternative spectral products to XSPEC work directory?  These include
# the PATTERN==0 only and the annular region source spectra (with associated
# backgrounds and matrices) that are useful when pileup becomes a problem.
#
set possiblePileUp = 0
#
# If this is > 0, then these alternative files are also copied into the XSPEC
# work directory; if <= 0 the data products are still generated but remain only
# in the OBSID/pps/ data directory.

### Disregard source spectra with exposure times less than this value:
#
set minExpTime = 100
#
# Spectra with smaller exposure times will still be generated and copied to
# XSPEC_WORKDIR, but will not be included in the XSPEC file setup scripts.
# This value should always be larger than 1.5 (sec)
# because a faulty observation is flagged with EXPOSURE = 1.0 and our test is
# whether EXPOSURE - minExpTime < 0 after rounding to the nearest integer...

### Don't bother processing spectra with BACKSCAL values lower than:
#
set minSrcArea = 400
#
# BACKSCAL = detector area of extraction region after factoring in dead pixels,
# chip boundaries, etc., in units of detector pixels (0.05" on a side, so 400 =
# 1 square arcsecond).  If an extracted source spectrum has BACKSCAL below this
# threshold, then no further processing will be done (notably, no response
# matrices or pileup-contingency spectra will be generated).

##### End of hard-coded parameters
##### End of hard-coded parameters
##### End of hard-coded parameters

##### Derived variables
#
setenv IAUNAME_TRUN \
   `echo ${IAUNAME_XMM} | tr -d ' ' | sed -e 's/2XMMiJ//g' -e 's/2XMMJ//g' | tr '+' 'p' | tr '-' 'm'`
# alternatively (less efficiently?):
# foreach i ($IAUNAME_XMM)
#    setenv IAUNAME_TRUN `echo ${i} | tr -d 'J' | tr '+' 'p' | tr '-' 'm'`
# end
#
setenv NAME_TRUN \
    `echo ${COMMONNAME} | tr -d ' ' | awk '{if (substr($0,12,1) == ".") {print substr($0,1,11)} else {print substr($0,1,12)}}' | tr '+' 'p' | tr '-' 'm'`

setenv XSPEC_WORKDIR ${AnalysisPath}${DETID_XMM}_${IAUNAME_TRUN}_${NAME_TRUN}

if (-e ${XSPEC_WORKDIR}) then
    #
    # If a version of the xspec work directory already exists,
    # preserve it by appending "_v#" to its name.
    # 
    # Test whether pre-existing folder is empty. 
    # If it is, reuse it; if not, rename using the lowest unused "_v#" extension:
    #
    set preExistFolderContents=`/bin/ls ${XSPEC_WORKDIR}`
    if ( "${preExistFolderContents}"x != x ) then
        set tmpCounter = 0
        set preExistFolder = 1
        while (${preExistFolder} == 1)
            @ tmpCounter++
            set altWorkFolder = ${XSPEC_WORKDIR}"_v"${tmpCounter}
            if (! -e ${altWorkFolder}) set preExistFolder = 0
        end
        mv ${XSPEC_WORKDIR} ${altWorkFolder}
        mkdir ${XSPEC_WORKDIR}
        unset altWorkFolder
        unset preExistFolder
        unset tmpCounter
    endif
    unset preExistFolderContents
else
    mkdir ${XSPEC_WORKDIR}
endif


# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (2) <><>"
# echo "<><>             retrieving archival XMM data (as needed) <><>"
echo "<><><><> ...setting up data... <><><><>"
#############
# (2) get ODF and PPS data
#

if ((`pwd` != $DataPath_XMM) && (`pwd`/ != $DataPath_XMM)) then
    pushd $DataPath_XMM
    setenv moveToDataPath
endif
${ScriptPath_XMM}fetchXMMobs.csh ${OBSID_XMM}


# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (2a) <><>"
# echo "<><>             setting up environment and directories    <><>"
echo " "
echo "<><><><> ...setting up environment and packages... <><><><>"
#############
# (2a) change directories and make sure SAS is set up to work with
# these data
#
pushd ${OBSID_XMM}/pps/

if (! $?XMMSASSET) then
    # Debug:
    # echo " " |& tee -a ${SasOutLog}
    # echo "<><><> We will now invoke SETSAS w/o arguments do an initial setup for SAS"
    #
    source ${ScriptPath_XMM}localsetsas.csh
    #
    # N.b. - if I invoke "localsetsas.csh" without using "source ...", then the
    # parameters it sets do not get inerited by the parent shell.
endif

if (! $?XMMSASSET) then
    echo "*** WARNING: For some reasons, SAS appears not to be set up   ***"
    echo "             even though the alias SETSAS has been invoked"
    echo "             Let's try this again..."
    source ${ScriptPath_XMM}localsetsas.csh
    #
    # Now that I use "source" in the previous invocation of setsas, I'd be very
    # surprised if there were times that this second invocation worked and the
    # first one did not...
endif

if (! $?XMMSASSET) then
    echo "***** ERROR: For some reasons, SAS is STILL not set up!     *****"
    echo "      I will now attempt to set the necessary parameters"
    echo "      one last time, but you'll want to carefully check"
    echo "      the logs for errors"
    if (! $?SAS_DIR) setenv SAS_DIR /soft/sas/xmmsas_current
    setenv SAS_CCFPATH /soft/sas/ccf
    setenv SAS_BROWSER open
    if (! $?DYLD_LIBRARY_PATH ) setenv DYLD_LIBRARY_PATH
    source ${ScriptPath_XMM}localsetsas.csh
    setenv XMMSASSET
endif
    
if (${UpdateCCF} == YES) then
    #
    echo " " |& tee -a ${SasOutLog}
    echo "<><><> Invoking UPDATESASCCF to make sure calibrations are current..." |& tee -a ${SasOutLog}
    echo " " |& tee -a ${SasOutLog}
    #
    updatesasccf
endif

setenv SAS_VERBOSITY ${SAS_VERBOSITY_DEFAULT}
setenv SAS_SUPPRESS_WARNING 1
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! ${SasOutLog}
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! ${SasOutLog}
echo "* * * SAS data prep done on  `date` * * *" >>! ${SasOutLog}
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! ${SasOutLog}
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! ${SasOutLog}

echo " " |& tee -a ${SasOutLog}
echo "<><><> Now invoking SETSAS with argument ../odf:" |& tee -a ${SasOutLog}
# # source ${ScriptPath_XMM}localsetsas.csh ../odf |& tee -a ${SasOutLog}
# ${ScriptPath_XMM}localsetsas.csh ../odf |& tee -a ${SasOutLog}
# source ${ScriptPath_XMM}localsetsas.csh ../odf ${SasOutLog} |& tee -a ${SasOutLog}
${ScriptPath_XMM}localsetsas.csh ../odf ${SasOutLog} |& tee -a ${SasOutLog}

echo "Debug: The following SAS-related parameters are now defined:" >>! ${SasOutLog}
setenv | grep SAS >>! ${SasOutLog}
echo " " >>! ${SasOutLog}
# echo "Debug: The following SAS-related parameters are now defined:" |& tee -a ${SasOutLog}
# setenv | grep SAS |& tee -a ${SasOutLog}
# echo " " |& tee -a ${SasOutLog}

if (! ${?SAS_ODF}) then
    echo "For some reason, SAS_ODF is not defined.  Fixing that..." |& tee -a ${SasOutLog}
    set sumfile = `/bin/ls -t *SUM.SAS`
    if ( "${sumfile}"x != x ) setenv SAS_ODF ${sumfile[1]}
    setenv | grep SAS_ODF= |& tee -a ${SasOutLog}
    unset sumfile
else
    if (! -e ${SAS_ODF}) then
	echo "SAS_ODF appears have retained a previous setting.  Fixing that..." |& tee -a ${SasOutLog}
	echo "        it had been ${SAS_ODF}." |& tee -a ${SasOutLog}
	set sumfile = `/bin/ls -t *SUM.SAS`
	if ( "${sumfile}"x != x ) setenv SAS_ODF ${sumfile[1]}
	unset sumfile
	echo "        it is now   ${SAS_ODF}." |& tee -a ${SasOutLog}
    endif
endif

if (! ${?SAS_CCF}) then
    echo " " |& tee -a ${SasOutLog}
    echo "For some reason, SAS_CCF is not defined.  Fixing that..." |& tee -a ${SasOutLog}
    set sumfile = `/bin/ls -t *.cif`
    if ( "${sumfile}"x != x ) setenv SAS_CCF ${sumfile[1]}
    setenv | grep SAS_CCF= |& tee -a ${SasOutLog}
    unset sumfile
endif

set PPSMESSAGElist = `/bin/ls P*${OBSID_XMM}PPSMSG*ASC \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
if ("${PPSMESSAGElist}"x == x) then
    echo " " |& tee -a ${SasOutLog}
    echo "***************" |& tee -a ${SasOutLog}
    echo "*** Warning ***" |& tee -a ${SasOutLog}
    echo "***************" |& tee -a ${SasOutLog}
    echo "    No PPS message files were found for this data set." |& tee -a ${SasOutLog}
    echo "    Check whether there are any usable EPIC data." |& tee -a ${SasOutLog}
    echo " " |& tee -a ${SasOutLog}
else
    setenv PPSMESSAGE ${PPSMESSAGElist[1]}
    if (-e ${PPSMESSAGE}) then
	echo " " |& tee -a ${SasOutLog}
	echo "The PPS pipeline reports the following (in ${PPSMESSAGE}):" |& tee -a ${SasOutLog}
	echo " " |& tee -a ${SasOutLog}
	grep "pile-up" ${PPSMESSAGE} |& tee -a ${SasOutLog}
	echo " " |& tee -a ${SasOutLog}
	grep "BKGDSCRN" ${PPSMESSAGE} |& tee -a ${SasOutLog}
	echo " " |& tee -a ${SasOutLog}
	grep "duration" ${PPSMESSAGE} |& tee -a ${SasOutLog}
    endif
    unsetenv PPSMESSAGE
endif
unset PPSMESSAGElist


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (2b) <><>" |& tee -a ${SasOutLog}
#echo "<><>             selecting events files                    <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...finding (and possibly generating) events list files... <><><><>" |& tee -a ${SasOutLog}
#############
# (2b) identify and if necessary generate the events files;
#      use the most recent one for each exposure with each instrument
#
gunzip -f *.FIT.gz |& sed "s/tcsh: gunzip: No match.//g" |& tee -a ${SasOutLog}
#  ...just in case I've already generated data products for this object...

# echo " " |& tee -a ${SasOutLog}
echo "Collecting information on events lists..." |& tee -a ${SasOutLog}

set PN_EVTlistPPS = `/bin/ls -t P${OBSID_XMM}PN*PIEVLI*.FTZ  \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
set PN_EVTlistMine = `/bin/ls -t P${OBSID_XMM}PN*PIEVLI*.FIT \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
set PN_EVTlistAll = `/bin/ls -t P${OBSID_XMM}PN*PIEVLI*.F*   \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
# N.b. - the newest event list is listed first when more than one is found...
# N.b. - extensions would be FTZ for PPS FITS products, FIT for mine...

if ( ("${PN_EVTlistAll}"x == x) || \
     ( ("${PN_EVTlistMine}"x == x) && ($ForceEvtExtract == YES) ) ) then
    # ...if there are no PN event files, or none extracted by me and I've
    #    chosen to attempt my own extraction in order to find any alternative
    #    exposures not provided by PPS...
    echo " " |& tee -a ${SasOutLog}
    echo "Using EPCHAIN to extract PN events list(s)..." |& tee -a ${SasOutLog}
    if (${SAS_VERBOSITY_DEFAULT} < 5) setenv SAS_VERBOSITY 5
    if (-e ep_out.log) /bin/rm -f ep_out.log
    epchain >&! ep_out.log
    ### Optional EPCHAIN parameters that may be useful:
    ###    retain events below 150eV using screenlowthresh=100 ("=0" retains all)
    ###    withpatplot=Y to run epatplot (but there's no way to save a QDP file)
    ###    note: cannot set verbosity using "-V 4" (unlike emchain...)
    echo " " >>! ${SasOutLog}
    echo "Output from EPCHAIN with SAS_VERBOSITY=${SAS_VERBOSITY}:" >>! ${SasOutLog}
    setenv SAS_VERBOSITY ${SAS_VERBOSITY_DEFAULT}
    cat ep_out.log >>! ${SasOutLog}
    #
    # echo "   Warnings and errors from epchain:"
    # grep "\*\*" ep_out.log
    echo "   Warnings reported by epchain:"
    grep -i " warning " ep_out.log
    echo "   Errors reported by epchain:"
    grep -i " error " ep_out.log
    #
    set PN_EVTlistMine = `/bin/ls -t P${OBSID_XMM}PN*PIEVLI*.FIT \
	|& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
endif

set tmpPN_EVTlist = ( ${PN_EVTlistMine} ${PN_EVTlistPPS} )
if ("${tmpPN_EVTlist}"x != x) then
    # ...if there are any PN event files, select the newest one for each exposure
    #    taken with a scientifically valid filter (prioritizing ones I generated)...
    source ${ScriptPath_XMM}chooseEVTlists.csh "${tmpPN_EVTlist}"
else
    set chosenEVTlist = ( )
endif
set PN_EVTlist = ( ${chosenEVTlist} )
unset chosenEVTlist
unset tmpPN_EVTlist
#
# PN_EVTlist should now contain one events file for every PN imaging mode
# exposure taken with a scientifically useful filter

echo " " |& tee -a ${SasOutLog}
if ("${PN_EVTlist}"x == x) then
    # ...if there are no good events lists from either me or PPS...
    setenv viablePNdata 0
    echo "*** Warning: no usable PN imaging event lists were found ***" |& tee -a ${SasOutLog}
    echo "    No further attempt is made to deal with PN for this observation" |& tee -a ${SasOutLog}
else
    setenv viablePNdata ${#PN_EVTlist}
    set PN_EXPID = ( ${PN_EVTlist} )
    set PN_FILTER = ( ${PN_EVTlist} )
    set PN_SUBMODE = ( ${PN_EVTlist} )
    if ("${PN_EVTlistPPS}"x != x) then
	set tmpCount1 = ${#PN_EVTlistPPS}
    else
	set tmpCount1 = 0
    endif
    if ("${PN_EVTlistMine}"x != x) then
	set tmpCount2 = ${#PN_EVTlistMine}
    else
	set tmpCount2 = 0
    endif
    echo "${tmpCount1} PN events files from the PPS pipeline and ${tmpCount2} that I produced are found," |& tee -a ${SasOutLog}
    echo "   of which $viablePNdata PN events file(s) are to be used:" |& tee -a ${SasOutLog}
    # echo "(file, exposure ID, filter, submode, start of obs, end of obs)" |& tee -a ${SasOutLog}
    set tmpCounter = 1
    while (${tmpCounter} <= ${#PN_EVTlist})
	fkeypar ${PN_EVTlist[${tmpCounter}]} EXPIDSTR
	set tmpEXPIDSTR = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${PN_EVTlist[${tmpCounter}]} FILTER
	set tmpFILTER = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${PN_EVTlist[${tmpCounter}]} SUBMODE
	set tmpSUBMODE = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${PN_EVTlist[${tmpCounter}]} DATE-OBS
	set tmpDATEOBS = `pget fkeypar value | tr -d "'" | tr -d '"'`
	# fkeypar ${PN_EVTlist[${tmpCounter}]} DATE-END
	# set tmpDATEEND = `pget fkeypar value | tr -d "'" | tr -d '"'`
	# echo "${PN_EVTlist[${tmpCounter}]}, ${tmpEXPIDSTR}, ${tmpFILTER}, ${tmpSUBMODE}, ${tmpDATEOBS}, ${tmpDATEEND}" |& tee -a ${SasOutLog}
	echo "${PN_EVTlist[${tmpCounter}]}, ${tmpEXPIDSTR}, ${tmpFILTER}, ${tmpSUBMODE}, ${tmpDATEOBS}" |& tee -a ${SasOutLog}
	set PN_EXPID[${tmpCounter}] = ${tmpEXPIDSTR}
	set PN_FILTER[${tmpCounter}] = ${tmpFILTER}
	set PN_SUBMODE[${tmpCounter}] = ${tmpSUBMODE}
        @ tmpCounter++
    end
    unset tmpCount1
    unset tmpCount2
    unset tmpCounter
    unset tmpEXPIDSTR
    unset tmpFILTER
    unset tmpSUBMODE
    unset tmpDATEOBS
    # unset tmpDATEEND
endif
unset PN_EVTlistPPS
unset PN_EVTlistMine

set M1_EVTlistPPS = `/bin/ls -t P${OBSID_XMM}M1*MIEVLI*.FTZ  \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
set M1_EVTlistMine = `/bin/ls -t P${OBSID_XMM}M1*MIEVLI*.FIT \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
set M1_EVTlistAll = `/bin/ls -t P${OBSID_XMM}M1*MIEVLI*.F*   \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`

if ( ("${M1_EVTlistAll}"x == x) || \
     ( ("${M1_EVTlistMine}"x == x) && ($ForceEvtExtract == YES) ) ) then
    # ...if there are no M1 event files, or none extracted by me and I've
    #    chosen to attempt my own extraction in order to find any alternative
    #    exposures not provided by PPS...
    echo " " |& tee -a ${SasOutLog}
    echo "Using EMCHAIN to extract M1 events list(s)..." |& tee -a ${SasOutLog}
    if (-e em_out.log) /bin/rm -f em_out.log
    if (${SAS_VERBOSITY_DEFAULT} < 4) then
	emchain instruments=m1 globalflare=N filteratt=Y filterhk=Y -V 4 >&! em_out.log
    else
	emchain instruments=m1 globalflare=N filteratt=Y filterhk=Y >&! em_out.log
    endif
    ### Optional parameters that may be useful:
    ###    MOS1 only: instruments=m1 
    ###    SAS_VERBOSITY = 4 is recommended: -V 4
    ###    how retain events below 150eV? "evselect:?????=100"? Don't think that works...
    ###    no equivalent to epchain "withpatplot=Y"
    ###    filteratt and filterhk cause HK and attitude GTI files to be generated & applied
    ###    makeflaregti=Y (default) to generate PPS flare GTIs and include as
    ###		extensions to EVT file
    ###    can change flare threshold from default 2.0 using (e.g.) flaremaxrate=3.0
    ### 	(typical background rate is 0.8)
    ###    applyflaregti=N because I will define my own flare GTI later
    ###		(if Y, flare GTI will only be applied if good time > 10% of exposure)
    #
    # Alternatively, we could use emproc:
    # emproc selectinstruments=yes emos1=yes |& tee -a ${SasOutLog}
    # but emproc produces data products with different nomenclature from PPS...
    #
    echo " " >>! ${SasOutLog}
    echo 'Output from "emchain instruments=m1 globalflare=N filteratt=Y filterhk=Y -V 4":' >>! ${SasOutLog}
    cat em_out.log >>! ${SasOutLog}
    #
    # echo "   Warnings and errors from emchain:"
    # grep "\*\*" em_out.log
    echo "   Warnings reported by emchain (for M1):"
    grep -i " warning " em_out.log
    echo "   Errors reported by emchain (for M1):"
    grep -i " error " em_out.log
    #
    set M1_EVTlistMine = `/bin/ls -t P${OBSID_XMM}M1*MIEVLI*.FIT \
	|& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
endif

set tmpM1_EVTlist = ( ${M1_EVTlistMine} ${M1_EVTlistPPS} )
if ("${tmpM1_EVTlist}"x != x) then
    # ...if there are any M1 event files, select the newest one for each exposure
    #    taken with a scientifically valid filter (prioritizing ones I generated)...
    source ${ScriptPath_XMM}chooseEVTlists.csh "${tmpM1_EVTlist}"
else
    set chosenEVTlist = ( )
endif
set M1_EVTlist = ( ${chosenEVTlist} )
unset chosenEVTlist
unset tmpM1_EVTlist
#
# M1_EVTlist should now contain one events file for every M1 imaging
# mode observation made using a scientifically useful filter

echo " " |& tee -a ${SasOutLog}
if ("${M1_EVTlist}"x == x) then
    # ...if there are no good events lists from either me or PPS...
    setenv viableM1data 0
    echo "*** Warning: no usable M1 imaging event lists were found ***" |& tee -a ${SasOutLog}
    echo "    No further attempt is made to deal with M1 for this observation" |& tee -a ${SasOutLog}
else
    setenv viableM1data ${#M1_EVTlist}
    set M1_EXPID = ( ${M1_EVTlist} )
    set M1_FILTER = ( ${M1_EVTlist} )
    set M1_SUBMODE = ( ${M1_EVTlist} )
    if ("${M1_EVTlistPPS}"x != x) then
	set tmpCount1 = ${#M1_EVTlistPPS}
    else
	set tmpCount1 = 0
    endif
    if ("${M1_EVTlistMine}"x != x) then
	set tmpCount2 = ${#M1_EVTlistMine}
    else
	set tmpCount2 = 0
    endif
    echo "${tmpCount1} M1 events files from the PPS pipeline and ${tmpCount2} that I produced are found," |& tee -a ${SasOutLog}
    echo "   of which $viableM1data M1 events file(s) are to be used:" |& tee -a ${SasOutLog}
    # echo "(file, exposure ID, filter, submode, start of obs, end of obs)" |& tee -a ${SasOutLog}
    set tmpCounter = 1
    while (${tmpCounter} <= ${#M1_EVTlist})
	fkeypar ${M1_EVTlist[${tmpCounter}]} EXPIDSTR
	set tmpEXPIDSTR = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${M1_EVTlist[${tmpCounter}]} FILTER
	set tmpFILTER = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${M1_EVTlist[${tmpCounter}]} SUBMODE
	set tmpSUBMODE = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${M1_EVTlist[${tmpCounter}]} DATE-OBS
	set tmpDATEOBS = `pget fkeypar value | tr -d "'" | tr -d '"'`
	# fkeypar ${M1_EVTlist[${tmpCounter}]} DATE-END
	# set tmpDATEEND = `pget fkeypar value | tr -d "'" | tr -d '"'`
	# echo "${M1_EVTlist[${tmpCounter}]}, ${tmpEXPIDSTR}, ${tmpFILTER}, ${tmpSUBMODE}, ${tmpDATEOBS}, ${tmpDATEEND}" |& tee -a ${SasOutLog}
	echo "${M1_EVTlist[${tmpCounter}]}, ${tmpEXPIDSTR}, ${tmpFILTER}, ${tmpSUBMODE}, ${tmpDATEOBS}" |& tee -a ${SasOutLog}
	set M1_EXPID[${tmpCounter}] = ${tmpEXPIDSTR}
	set M1_FILTER[${tmpCounter}] = ${tmpFILTER}
	set M1_SUBMODE[${tmpCounter}] = ${tmpSUBMODE}
        @ tmpCounter++
    end
    unset tmpCount1
    unset tmpCount2
    unset tmpCounter
    unset tmpEXPIDSTR
    unset tmpFILTER
    unset tmpSUBMODE
    unset tmpDATEOBS
    # unset tmpDATEEND
endif
unset M1_EVTlistPPS
unset M1_EVTlistMine

set M2_EVTlistPPS = `/bin/ls -t P${OBSID_XMM}M2*MIEVLI*.FTZ  \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
set M2_EVTlistMine = `/bin/ls -t P${OBSID_XMM}M2*MIEVLI*.FIT \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
set M2_EVTlistAll = `/bin/ls -t P${OBSID_XMM}M2*MIEVLI*.F*   \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`

if ( ("${M2_EVTlistAll}"x == x) || \
     ( ("${M2_EVTlistMine}"x == x) && ($ForceEvtExtract == YES) ) ) then
    # ...if there are no M2 event files, or none extracted by me and I've
    #    chosen to attempt my own extraction in order to find any alternative
    #    exposures not provided by PPS...
    echo " " |& tee -a ${SasOutLog}
    echo "Using EMCHAIN to extract M2 events list(s)..." |& tee -a ${SasOutLog}
    if (-e out.log) /bin/rm -f out.log
    if (${SAS_VERBOSITY_DEFAULT} < 4) then
	emchain instruments=m2 globalflare=N filteratt=Y filterhk=Y -V 4 >&! out.log
    else
	emchain instruments=m2 globalflare=N filteratt=Y filterhk=Y >&! out.log
    endif
    ### Optional parameters that may be useful:
    ###    MOS2 only: instruments=m2 
    ###    SAS_VERBOSITY = 4 is recommended: -V 4
    ###    how retain events below 150eV? "evselect:?????=100"? Don't think that works...
    ###    no equivalent to epchain "withpatplot=Y"
    ###    filteratt and filterhk cause HK and attitude GTI files to be generated & applied
    ###    makeflaregti=Y (default) to generate PPS flare GTIs and include as
    ###		extensions to EVT file
    ###    can change flare threshold from default 2.0 using (e.g.) flaremaxrate=3.0
    ### 	(typical background rate is 0.8)
    ###    applyflaregti=N because I will define my own flare GTI later
    ###		(if Y, flare GTI will only be applied if good time > 10% of exposure)
    #
    # Alternatively, we could use emproc:
    # emproc selectinstruments=yes emos2=yes |& tee -a ${SasOutLog}
    # but emproc produces data products with different nomenclature from PPS...
    #
    echo " " >>! ${SasOutLog}
    echo 'Output from "emchain instruments=m2 globalflare=N filteratt=Y filterhk=Y -V 4":' >>! ${SasOutLog}
    cat out.log >>! ${SasOutLog}
    #
    # echo "   Warnings and errors from emchain:"
    # grep "\*\*" out.log
    echo "   Warnings reported by emchain (for M2):"
    grep -i " warning " out.log
    echo "   Errors reported by emchain (for M2):"
    grep -i " error " out.log
    set M2_EVTlistMine = `/bin/ls -t P${OBSID_XMM}M2*MIEVLI*.FIT \
	|& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
endif

if (-e out.log) /bin/rm out.log

set tmpM2_EVTlist = ( ${M2_EVTlistMine} ${M2_EVTlistPPS} )
if ("${tmpM2_EVTlist}"x != x) then
    # ...if there are any M2 event files, select the newest one for each exposure
    #    taken with a scientifically valid filter (prioritizing ones I generated)...
    source ${ScriptPath_XMM}chooseEVTlists.csh "${tmpM2_EVTlist}"
else
    set chosenEVTlist = ( )
endif
set M2_EVTlist = ( ${chosenEVTlist} )
unset chosenEVTlist
unset tmpM2_EVTlist
#
# M2_EVTlist should now contain one events file for every M2 imaging
# mode observation made using a scientifically useful filter

echo " " |& tee -a ${SasOutLog}
if ("${M2_EVTlist}"x == x) then
    # ...if there are no good events lists from either me or PPS...
    setenv viableM2data 0
    echo "*** Warning: no usable M2 imaging event lists were found ***" |& tee -a ${SasOutLog}
    echo "    No further attempt is made to deal with M2 for this observation" |& tee -a ${SasOutLog}
else
    setenv viableM2data ${#M2_EVTlist}
    set M2_EXPID = ( ${M2_EVTlist} )
    set M2_FILTER = ( ${M2_EVTlist} )
    set M2_SUBMODE = ( ${M2_EVTlist} )
    if ("${M2_EVTlistPPS}"x != x) then
	set tmpCount1 = ${#M2_EVTlistPPS}
    else
	set tmpCount1 = 0
    endif
    if ("${M2_EVTlistMine}"x != x) then
	set tmpCount2 = ${#M2_EVTlistMine}
    else
	set tmpCount2 = 0
    endif
    echo "${tmpCount1} M2 events files from the PPS pipeline and ${tmpCount2} that I produced are found," |& tee -a ${SasOutLog}
    echo "   of which $viableM2data M2 events file(s) are to be used:" |& tee -a ${SasOutLog}
    # echo "(file, exposure ID, filter, submode, start of obs, end of obs)" |& tee -a ${SasOutLog}
    set tmpCounter = 1
    while (${tmpCounter} <= ${#M2_EVTlist})
	fkeypar ${M2_EVTlist[${tmpCounter}]} EXPIDSTR
	set tmpEXPIDSTR = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${M2_EVTlist[${tmpCounter}]} FILTER
	set tmpFILTER = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${M2_EVTlist[${tmpCounter}]} SUBMODE
	set tmpSUBMODE = `pget fkeypar value | tr -d "'" | tr -d '"'`
	fkeypar ${M2_EVTlist[${tmpCounter}]} DATE-OBS
	set tmpDATEOBS = `pget fkeypar value | tr -d "'" | tr -d '"'`
	# fkeypar ${M2_EVTlist[${tmpCounter}]} DATE-END
	# set tmpDATEEND = `pget fkeypar value | tr -d "'" | tr -d '"'`
	# echo "${M2_EVTlist[${tmpCounter}]}, ${tmpEXPIDSTR}, ${tmpFILTER}, ${tmpSUBMODE}, ${tmpDATEOBS}, ${tmpDATEEND}" |& tee -a ${SasOutLog}
	echo "${M2_EVTlist[${tmpCounter}]}, ${tmpEXPIDSTR}, ${tmpFILTER}, ${tmpSUBMODE}, ${tmpDATEOBS}" |& tee -a ${SasOutLog}
	set M2_EXPID[${tmpCounter}] = ${tmpEXPIDSTR}
	set M2_FILTER[${tmpCounter}] = ${tmpFILTER}
	set M2_SUBMODE[${tmpCounter}] = ${tmpSUBMODE}
        @ tmpCounter++
    end
    unset tmpCount1
    unset tmpCount2
    unset tmpCounter
    unset tmpEXPIDSTR
    unset tmpFILTER
    unset tmpSUBMODE
    unset tmpDATEOBS
    # unset tmpDATEEND
endif
unset M2_EVTlistPPS
unset M2_EVTlistMine

set EPIC_TIEVLIlist = `/bin/ls *TIEVLI*.F* \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
if ("${EPIC_TIEVLIlist}"x != x) then
    echo "Note that the following timing mode events files were found:" |& tee -a ${SasOutLog}
    /bin/ls -l *TIEVLI*.F* |& tee -a ${SasOutLog}
    echo " " |& tee -a ${SasOutLog}
    echo "These have been ignored." |& tee -a ${SasOutLog}
endif
unset EPIC_TIEVLIlist

# At this point the following vectors are defined:
#
# [INST]_EVTlist  	consisting of 1 imaging event list for each usable exposure
# [INST]_EXPID    	listing the EXPIDSTR value for each of these exposures
# [INST]_FILTER   	listing the FILTER used for each of these exposures
# [INST]_SUBMODE  	listing the SUBMODE windowing mode used for each exposure
#
# [INST] is one of PN, M1 or M2
# The sequence of EXPIDSTR, FILTER and SUBMODE values corresponds to the
# sequence of event file entries in [INST]_EVTlist


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (3)  <><>" |& tee -a ${SasOutLog}
#echo "<><>             determining which spectra are needed      <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...determining which spectra are needed... <><><><>" |& tee -a ${SasOutLog}
#############
# (3) check whether previously-extracted spectra exist for all three detectors
#
if ($UsePPSspectra == YES) then
    setenv specFITSsuffix F
else
    setenv specFITSsuffix FIT
endif

set SPEClist = ( )	# Defined here to ensure they exist when I use unset...
set extractSPEC = ( )

if ($viablePNdata > 0) then
    source ${ScriptPath_XMM}setSPEClist.csh "$PN_EVTlist" "$PN_EXPID" PN $OBSID_XMM $SRCNUM_XMM $specFITSsuffix ${SasOutLog}
endif
set PNSPEClist = ( ${SPEClist} )
set extractPNSPEC  = ( ${extractSPEC} )

if ($viableM1data > 0) then
    source ${ScriptPath_XMM}setSPEClist.csh "$M1_EVTlist" "$M1_EXPID" M1 $OBSID_XMM $SRCNUM_XMM $specFITSsuffix ${SasOutLog}
endif
set M1SPEClist = ( ${SPEClist} )
set extractM1SPEC  = ( ${extractSPEC} )

if ($viableM2data > 0) then
    source ${ScriptPath_XMM}setSPEClist.csh "$M2_EVTlist" "$M2_EXPID" M2 $OBSID_XMM $SRCNUM_XMM $specFITSsuffix ${SasOutLog}
endif
set M2SPEClist = ( ${SPEClist} )
set extractM2SPEC  = ( ${extractSPEC} )

unset SPEClist
unset extractSPEC

# At this point, each entry in the [INST]SPEClist vectors is either an existing
# spectral file name (choosing the newest when more than one is available for a
# given exposure, excluding PPS pipeline spectra if $UsePPSspectra != "YES") or
# the name of a spectrum to be extracted.  Names are defined only when there are
# usable imaging events lists.  If no such events lists are available for a
# given instrument (neither from PPS or generated by me), then this vector is
# defined but empty.
#
# The extract[INST]SPEC vector entries should all be either 1 or 0; 1 if the
# source spectrum needs to be extracted and 0 if it already exists.  Note,
# however, that at present we have not tested whether we need to extract the
# background spectrum or specialized source spectra (those with the PSF core
# excised or with only grade 0 events), so extracting may still be needed (and
# hence this flag is useless?).
#
# No list has yet been made to include file names for the background spectra or
# the source spectra with special selection parameters (excising the PSF core
# or choosing only PATTERN==0 -- i.e., single -- events).


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (4)  <><>" |& tee -a ${SasOutLog}
#echo "<><>             defining filters and making images        <><>" |& tee -a ${SasOutLog}
#############
# (4) visually inspect extraction regions to look for contaminating
# sources in either the source or background extraction regions
#
# Automated processing will assume that default regions are OK, but we
# should save images with the extraction regions overlaid for later
# inspection/confirmation.
#
# Steps taken:
#   - (a) define list of images (some of which likely will require extraction)
#   - (b) define background flare GTIs for each exposure with each instrument
#   - (b) extract any missing images, making use of flare GTI filters
#   - (c) ensure that source and background region filters and files are
#     available for each exposure
#   - (c) save JPG and EPS images with extraction regions overlaid for later inspection

#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (4a) <><>" |& tee -a ${SasOutLog}
#echo "<><>             make list of image files to use           <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...getting images and defining background flare GTIs... <><><><>" |& tee -a ${SasOutLog}
#############
# (4a) First verify that full-band (and *customized* PN band 1, hereafter
# labeled as band a) images exist.  If they do not, then define names for the
# new images to be extracted and set the extraction flags to 1...

if ($UsePPSimages == YES) then
    setenv imageFITSsuffix F
else
    setenv imageFITSsuffix FIT
endif

set IMAGElist = ( )	# Defined here to ensure they exist when I use unset...
set extractIMAGE = ( )

if ($viablePNdata > 0) then
    source ${ScriptPath_XMM}setIMAGElist.csh "$PN_EVTlist" "$PN_EXPID" PN $OBSID_XMM 8 ${imageFITSsuffix} ${SasOutLog}
endif
set PNimage8list = ( ${IMAGElist} )
set extractPNimage8 = ( ${extractIMAGE} )

if ($viablePNdata > 0) then
    source ${ScriptPath_XMM}setIMAGElist.csh "$PN_EVTlist" "$PN_EXPID" PN $OBSID_XMM a ${imageFITSsuffix} ${SasOutLog}
    # Note: for PN band 1 images, we require our own extraction because we are
    # interested in using this image to check for low energy noise.  This noise
    # is most apparent in double events with energies <350 eV but the PPS band 1
    # images include only singles.  We therefore label the band of the image as "a"
endif
set PNimage1list = ( ${IMAGElist} )
set extractPNimage1 = ( ${extractIMAGE} )

if ($viableM1data > 0) then
    source ${ScriptPath_XMM}setIMAGElist.csh "$M1_EVTlist" "$M1_EXPID" M1 $OBSID_XMM 8 ${imageFITSsuffix} ${SasOutLog}
endif
set M1image8list = ( ${IMAGElist} )
set extractM1image8 = ( ${extractIMAGE} )

if ($viableM2data > 0) then
    source ${ScriptPath_XMM}setIMAGElist.csh "$M2_EVTlist" "$M2_EXPID" M2 $OBSID_XMM 8 ${imageFITSsuffix} ${SasOutLog}
endif
set M2image8list = ( ${IMAGElist} )
set extractM2image8 = ( ${extractIMAGE} )

unset IMAGElist
unset extractIMAGE

echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (4b) <><>" |& tee -a ${SasOutLog}
#echo "<><>             define background flare GTIs and          <><>" |& tee -a ${SasOutLog}
#echo "<><>             extract any images necessary              <><>" |& tee -a ${SasOutLog}
#############
# (4b) Now, ensure that background flare GTIs are available for all exposures
# and extract any missing images (making use of the GTIs when so doing)...

### Notes on defining the background flare GTI:
###
### PPS processing defines background flare GTIs by extracting light curves
### from source-free parts of the field and applies a maximum count rate
### threshold of 2.0 (MOS) or 10.0 (PN) ct/ks/arcmin^2.  These GTIs are applied
### unless they exclude >90% of the available exposure time, in which case the
### instrument is skipped by the latter stages of the PPS pipeline (i.e., no
### source detection or source-specific data products).
###
### The maximum count rate thresholds used by PPS are reasonably conservative
### (typical background count rates are 0.8 for MOS or a few for PN), but there
### may be instances in which the non-flaring background level is higher.  For
### example, the non-flaring background of OBSID 0051760101 (PG 1244+026) hovers
### between 2 and 3 for the MOS; the PPS GTI threshold eliminates >98% of the
### exposure time whereas a threshold of 3.0 would retain roughly half of the
### exposure.  Consequently, PPS does not apply flare GTIs and therefore does
### not produce any MOS spectra for this observation, even though there could be
### viable data.  We favor a different approach, although we preserve the
### ability to define GTIs based upon arbitrary background count rates.
### 
### By default, this script uses the routine ESPFILT.  This routine has the
### advantage of not using an arbitrary high background threshold value.  It has
### the disadvantages of always rejecting the higher-background intervals, even
### if there are no flares, and of taking more time to run than simply using a
### fixed threshold.  Alternatively, TABGTIGEN may be used with a fixed
### threshold.  To choose between these alternatives, reset the values of
### ${PNbgdFlareGTItest} and ${MOSbgdFlareGTItest}.
###
### A third alternative, not implemented below, would be to reuse the same
### filters applied by PPS.  This would include background flare GTIs with the
### default fixed thresholds whenever these GTIs include at least 10% of the
### exposure time.  As suggested by Matthias Ehle at the SAS workshop, one could
### use:
###    evselect table=inevlist.fits filtertype=dataSubspace dssblock=image.fits:PRIMARY
###    keepfilteroutput=yes withfilteredset=yes filteredset=outevlist.fits
### This requires that a PPS-produced image file is avalable to be used as the
### input of the dssblock parameter.
###
### SAS suggestion:
### Ideally, I'd like to see a new version of ESPFILT with greater flexibility.
### My wishlist would include (1) instead of allowing count rates that are close
### to some (fitted) baseline value, limited to a range that appears to be
### something like +/- 1.5 sigmas, allow the number of sigmas to be set by the
### user (e.g., to be more inclusive, a user might want to only eliminate count
### rates that differ by 3 or 5 sigmas); (2) add a "minimum threshold" parameter
### that would be a minimum value to be used for the upper count rate cutoff
### when defining GTIs (i.e., eliminate time intervals when the count rate
### exceeds the larger of either the input fixed value or the upper threshold
### defined from the actual count rate distribution -- this will minimize the
### loss of usable data when the background is more than 1-2 sigma above the
### baseline but is still low, making it possible to retain 100% of the time
### interval when no flares take place...)

### Notes regarding the image extraction:
### 
### There is usually extra noise amongst the events close to the low energy
### threshold of real events (single events under ~200 eV, doubles under ~400 eV
### and triples under ~600 eV), so such events are generally avoided in order to
### make prettier pictures.  However, the low energy noise that is sometimes
### found at the low RawY edges of the PN is most obvious in doubles at energies
### <~ 350 eV, so we need to extract our own version of the band 1 images that
### include doubles in order to inspect for this noise.  To distinguish this
### from the PPS band 1 images we label these as "band a".
###
### Resolved question: Is #XMMEA_EP the best flag filters to use for PN images? 
### The PPS-generated images use something else.
### 
### Short answer: #XMMEA_EP is fine and should be sufficient.
###
### Somewhat longer answer: After testing my options, I decided to use slight
### variations on #XMMEA_EP and #XMMEA_EM when (re-)extracting images.  For the
### MOS I use ((FLAG & 0x766aa000)==0), which is the same as #XMMEA_EM except
### that I allow OUT_OF_FOV events.  For the PN I use ((FLAG & 0xfa0009)==0),
### which adds INVALID_PATTERN and ON_OFFSET_COLUMN to the events excluded by
### #XMMEA_EP.
###
### Long answer: Comparing #XMMEA_EP and the flags used by PPS, it seems that
### both are filtered with Flare and HK GTIs (PPS does this explicitly; my event
### file is pre-filtered with HK GTIs).  Both use "RAWY > 12".  Where we differ
### is the flag filters and energy intervals.  PPS uses three energy bands:
###   (200:500]:    PATTERN==0 && (FLAG & 0x2fa002c)==0
###   (500:1000]:   PATTERN<=4 && (FLAG & 0x2fa002c)==0
###   (1000:12000]: PATTERN<=4 && (FLAG & 0x2fa0024)==0
### What difference does the changed flag in the 4th bit (i.e., 8 in the units
### column) make?  Why is this used?  How do these flags compare with the
### #XMMEA_EP flag I apply universally?
###
### N.b.: (,) = exclusive limits, [,] = inclusive; thus, "(200:500]" = "200<E<=500"
###
### Section 3.1.1 (d) of the 2XMM Catalogue documentation
### ( http://xmmssc-www.star.le.ac.uk/Catalogue/2XMM/UserGuide_xmmcat.html )
### notes that PN images omit events with the "ON_OFFSET_COLUMN" flag set.  The
### individual band images (bands 1-5) also filter the "OUT_OF_FOV" flag, which
### is why they don't have the PN corners.  (For MOS, there are no PATTERN or
### FLAG selections).  No mention is made of changed FLAG settings for data >1 keV.
### 
### After much digging, I found an index of EPIC flag values (thanks to a Google
### search, which turned this up on a supplemental slide from a PPT file at
### http://www2.le.ac.uk/departments/physics-and-astronomy/research/src/Missions/xmm-newton/technical/mallorca-2007-11/Carter_bg.ppt )
### (I later learned that this same index is included in the header of the
### EVENTS extension of every XMM EPIC events file.  Who knew?)
###
### The difference is apparently that the <1 keV data are additionally filtered
### to eliminate data flagged as "ON_OFFSET_COLUMN" (whatever that means).
###
### The full set of flags used to make PPS images are:
### "CLOSE_TO_CCD_WINDOW" (0x4), "ON_OFFSET_COLUMN" (0x8),
### "CLOSE_TO_ONBOARD_BADPIX" (0x20), "IN_BAD_FRAME" (0x20000), "COSMIC_RAY"
### (0x80000), "MIP_ASSOCIATED" (0x100000), "ON_BADPIX" (0x200000), "SECONDARY"
### (0x400000), "TRAILING" (0x800000), and "OUT_OF_CCD_WINDOW" (0x2000000). 
###
### "XMMEA_EP" applies most of these filters, with the exceptions of
### "CLOSE_TO_CCD_WINDOW", "ON_OFFSET_COLUMN", "CLOSE_TO_ONBOARD_BADPIX" and
### "OUT_OF_CCD_WINDOW".

set tmpCounter = 1
set PN_FlareGTI = ( ${PN_EVTlist} )
while ($tmpCounter <= $viablePNdata)
    # For each PN imaging events list, start by defining the low background GTIs...
    #
    if (-e bgdFlareGTI.out) /bin/rm -f bgdFlareGTI.out
    source ${ScriptPath_XMM}bgdFlareGTI.csh ${PN_EVTlist[$tmpCounter]} ${PNbgdFlareGTItest} |& tee -a ${SasOutLog}
    #
    # To define GTIs using a fixed maximum background count rate, redefine
    #    PNbgdFlareGTItest with the desired (positive, real) threshold value...
    #
    # Amongst the outputs from bgdFlareGTI.csh is a GTI file that can be used
    # to extract the images; the script *should* point $OUT_FLARE_GTI at it...
    #
    # Any SAS verbose output has been stored in bgdFlareGTI.out.  Append this to
    # SasOutLog and highlight any warnings or errors:
    #
    if (-e bgdFlareGTI.out) then
	echo " " >>! ${SasOutLog}
	echo "   Output from SAS tasks called as a result of invoking" >>! ${SasOutLog}
	echo "   bgdFlareGTI.csh ${PN_EVTlist[$tmpCounter]} ${PNbgdFlareGTItest}" >>! ${SasOutLog}
	echo "   is as follows:" >>! ${SasOutLog}
	echo " " >>! ${SasOutLog}
	cat bgdFlareGTI.out >>! ${SasOutLog}
	echo " " >>! ${SasOutLog}
	echo "   If ESPFILT or TABGTIGEN generated any warnings or errors," |& tee -a ${SasOutLog}
	echo "   these appear on the following line(s):" |& tee -a ${SasOutLog}
	# grep "\*\*" bgdFlareGTI.out |& tee -a ${SasOutLog}
	grep -i " warning " bgdFlareGTI.out |& tee -a ${SasOutLog}
	grep -i " error " bgdFlareGTI.out |& tee -a ${SasOutLog}
	/bin/rm -f bgdFlareGTI.out
    endif
    #
    if (! ${?OUT_FLARE_GTI}) then
	set tempFileList = `/bin/ls -t *PN${PN_EXPID[$tmpCounter]}FBKGTI*.F* *PN${PN_EXPID[$tmpCounter]}-gti*.F*`
	setenv OUT_FLARE_GTI ${tempFileList[1]}
	unset tempFileList
    endif
    set PN_FlareGTI[$tmpCounter] = ${OUT_FLARE_GTI}
    #
    # ...then, extract image(s) using the background flare GTI filter:
    #
    if (-e ${OUT_FLARE_GTI}) then
	if (-e tmp.out) /bin/rm -f tmp.out
	if (${extractPNimage8[$tmpCounter]} > 0) then
	    evselect table=${PN_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${PNimage8list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0xfa0009)==0) && gti(${OUT_FLARE_GTI},TIME) && (((PI in [200:500)) && (PATTERN==0)) || ((PI in [500:12000]) && (PATTERN<=4))) && (RAWY > 12)" \
		>>&! tmp.out
		# could use "#XMMEA_EP" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${PNimage8list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
	if (${extractPNimage1[$tmpCounter]} > 0) then
	    evselect table=${PN_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${PNimage1list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0xfa0009)==0) && gti(${OUT_FLARE_GTI},TIME) && (PI in [200:500]) && (PATTERN<=4) && (RAWY > 12)" \
		>>&! tmp.out
		# could use "#XMMEA_EP" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${PNimage1list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
    else
	echo "*** Warning: no PN background flare GTI file was found," |& tee -a ${SasOutLog}
	echo "    so the newly-generated image(s) are not flare filtered ***" |& tee -a ${SasOutLog}
	if (-e tmp.out) /bin/rm -f tmp.out
	if (${extractPNimage8[$tmpCounter]} > 0) then
	    evselect table=${PN_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${PNimage8list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0xfa0009)==0) && (((PI in [200:500)) && (PATTERN==0)) || ((PI in [500:12000]) && (PATTERN<=4))) && (RAWY > 12)" \
		>>&! tmp.out
		# could use "#XMMEA_EP" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${PNimage8list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
	if (${extractPNimage1[$tmpCounter]} > 0) then
	    evselect table=${PN_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${PNimage1list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0xfa0009)==0) && (PI in [200:500]) && (PATTERN<=4) && (RAWY > 12)" \
		>>&! tmp.out
		# could use "#XMMEA_EP" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${PNimage1list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
    endif
    unsetenv OUT_FLARE_GTI
    @ tmpCounter++
end

set tmpCounter = 1
set M1_FlareGTI = ( ${M1_EVTlist} )
while ($tmpCounter <= $viableM1data)
    # For each M1 imaging events list, start by defining the low background GTIs...
    #
    if (-e bgdFlareGTI.out) /bin/rm -f bgdFlareGTI.out
    source ${ScriptPath_XMM}bgdFlareGTI.csh ${M1_EVTlist[$tmpCounter]} ${MOSbgdFlareGTItest} |& tee -a ${SasOutLog}
    #
    # To define GTIs using a fixed maximum background count rate, redefine
    #    MOSbgdFlareGTItest with the desired (positive, real) threshold value...
    #
    # Amongst the outputs from bgdFlareGTI.csh is a GTI file that can be used
    # to extract the images; the script *should* point $OUT_FLARE_GTI at it...
    #
    # Any SAS verbose output has been stored in bgdFlareGTI.out.  Append this to
    # SasOutLog and highlight any warnings or errors:
    #
    if (-e bgdFlareGTI.out) then
	echo "   Output from SAS tasks called as a result of invoking" >>! ${SasOutLog}
	echo "   bgdFlareGTI.csh ${M1_EVTlist[$tmpCounter]} ${MOSbgdFlareGTItest}" >>! $SasOutLog
	echo "   is as follows:" >>! ${SasOutLog}
	echo " " >>! ${SasOutLog}
	cat bgdFlareGTI.out >>! ${SasOutLog}
	echo " " >>! ${SasOutLog}
	echo "   If ESPFILT or TABGTIGEN generated any warnings or errors," |& tee -a ${SasOutLog}
	echo "   these appear on the following line(s):" |& tee -a ${SasOutLog}
	# grep "\*\*" bgdFlareGTI.out |& tee -a ${SasOutLog}
	grep -i " warning " bgdFlareGTI.out |& tee -a ${SasOutLog}
	grep -i " error " bgdFlareGTI.out |& tee -a ${SasOutLog}
	/bin/rm -f bgdFlareGTI.out
    endif
    #
    if (! ${?OUT_FLARE_GTI}) then
	set tempFileList = `/bin/ls -t *M1${M1_EXPID[$tmpCounter]}FBKGTI*.F* *M1${M1_EXPID[$tmpCounter]}-gti*.F*`
	setenv OUT_FLARE_GTI ${tempFileList[1]}
	unset tempFileList
    endif
    set M1_FlareGTI[$tmpCounter] = ${OUT_FLARE_GTI}
    #
    # ...then, extract image(s) using the background flare GTI filter:
    #
    if (-e ${OUT_FLARE_GTI}) then
	if (-e tmp.out) /bin/rm -f tmp.out
	if (${extractM1image8[$tmpCounter]} > 0) then
	    evselect table=${M1_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${M1image8list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0x766aa000)==0) && gti(${OUT_FLARE_GTI},TIME) && (((PI in [200:500)) && (PATTERN==0)) || ((PI in [500:700)) && (PATTERN<=4)) || ((PI in [700:12000]) && (PATTERN <= 12)))" \
		>>&! tmp.out
		# could use "#XMMEA_EM" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${M1image8list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
    else
	echo "*** Warning: no M1 background flare GTI file was found," |& tee -a ${SasOutLog}
	echo "    so the newly-generated image(s) are not flare filtered ***" |& tee -a ${SasOutLog}
	if (-e tmp.out) /bin/rm -f tmp.out
	if (${extractM1image8[$tmpCounter]} > 0) then
	    evselect table=${M1_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${M1image8list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0x766aa000)==0) && (((PI in [200:500)) && (PATTERN==0)) || ((PI in [500:700)) && (PATTERN<=4)) || ((PI in [700:12000]) && (PATTERN <= 12)))" \
		>>&! tmp.out
		# could use "#XMMEA_EM" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${M1image8list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
    endif
    unsetenv OUT_FLARE_GTI
    @ tmpCounter++
end

set tmpCounter = 1
set M2_FlareGTI = ( ${M2_EVTlist} )
while ($tmpCounter <= $viableM2data)
    # For each M2 imaging events list, start by defining the low background GTIs...
    #
    if (-e bgdFlareGTI.out) /bin/rm -f bgdFlareGTI.out
    source ${ScriptPath_XMM}bgdFlareGTI.csh ${M2_EVTlist[$tmpCounter]} ${MOSbgdFlareGTItest} |& tee -a ${SasOutLog}
    #
    # To define GTIs using a fixed maximum background count rate, redefine
    #    MOSbgdFlareGTItest with the desired (positive, real) threshold value...
    #
    # Amongst the outputs from bgdFlareGTI.csh is a GTI file that can be used
    # to extract the images; the script *should* point $OUT_FLARE_GTI at it...
    #
    # Any SAS verbose output has been stored in bgdFlareGTI.out.  Append this to
    # SasOutLog and highlight any warnings or errors:
    #
    if (-e bgdFlareGTI.out) then
	echo "   Output from SAS tasks called as a result of invoking" >>! ${SasOutLog}
	echo "   bgdFlareGTI.csh ${M2_EVTlist[$tmpCounter]} ${MOSbgdFlareGTItest}" >>! $SasOutLog
	echo "   is as follows:" >>! ${SasOutLog}
	echo " " >>! ${SasOutLog}
	cat bgdFlareGTI.out >>! ${SasOutLog}
	echo " " >>! ${SasOutLog}
	echo "   If ESPFILT or TABGTIGEN generated any warnings or errors," |& tee -a ${SasOutLog}
	echo "   these appear on the following line(s):" |& tee -a ${SasOutLog}
	# grep "\*\*" bgdFlareGTI.out |& tee -a ${SasOutLog}
	grep -i " warning " bgdFlareGTI.out |& tee -a ${SasOutLog}
	grep -i " error " bgdFlareGTI.out |& tee -a ${SasOutLog}
	/bin/rm -f bgdFlareGTI.out
    endif
    #
    if (! ${?OUT_FLARE_GTI}) then
	set tempFileList = `/bin/ls -t *M2${M2_EXPID[$tmpCounter]}FBKGTI*.F* *M2${M2_EXPID[$tmpCounter]}-gti*.F*`
	setenv OUT_FLARE_GTI ${tempFileList[1]}
	unset tempFileList
    endif
    set M2_FlareGTI[$tmpCounter] = ${OUT_FLARE_GTI}
    #
    # ...then, extract image(s) using the background flare GTI filter:
    #
    if (-e ${OUT_FLARE_GTI}) then
	if (-e tmp.out) /bin/rm -f tmp.out
	if (${extractM2image8[$tmpCounter]} > 0) then
	    evselect table=${M2_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${M2image8list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0x766aa000)==0) && gti(${OUT_FLARE_GTI},TIME) && (((PI in [200:500)) && (PATTERN==0)) || ((PI in [500:700)) && (PATTERN<=4)) || ((PI in [700:12000]) && (PATTERN <= 12)))" \
		>>&! tmp.out
		# could use "#XMMEA_EM" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${M2image8list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
    else
	echo "*** Warning: no M2 background flare GTI file was found," |& tee -a ${SasOutLog}
	echo "    so the newly-generated image(s) are not flare filtered ***" |& tee -a ${SasOutLog}
	if (-e tmp.out) /bin/rm -f tmp.out
	if (${extractM2image8[$tmpCounter]} > 0) then
	    evselect table=${M2_EVTlist[$tmpCounter]} imagebinning=binSize withimageset=yes \
		imageset=${M2image8list[$tmpCounter]} xcolumn=X ycolumn=Y \
		ximagebinsize=80 yimagebinsize=80 \
		expression="((FLAG & 0x766aa000)==0) && (((PI in [200:500)) && (PATTERN==0)) || ((PI in [500:700)) && (PATTERN<=4)) || ((PI in [700:12000]) && (PATTERN <= 12)))" \
		>>&! tmp.out
		# could use "#XMMEA_EM" instead of "(FLAG & ..."
	    cat tmp.out >>! ${SasOutLog}
	    echo "   Below are any errors or warnings that happened when creating ${M2image8list[$tmpCounter]}:" |& tee -a ${SasOutLog}
	    # grep "\*\*" tmp.out |& tee -a ${SasOutLog}
	    grep -i " warning " tmp.out |& tee -a ${SasOutLog}
	    grep -i " error " tmp.out |& tee -a ${SasOutLog}
	    /bin/rm -f tmp.out
	endif
    endif
    unsetenv OUT_FLARE_GTI
    @ tmpCounter++
end
unset tmpCounter

# At this point, all of the images that could be created should exist...

# Before moving on, generate *.gif and *.ps files demonstrating the light curves
# and the GITs:
echo " " |& tee -a ${SasOutLog}
echo "About to generate PS and GIF versions of light curve plots." |& tee -a ${SasOutLog}
echo "    If any errors follow this line, these will be from QDP." |& tee -a ${SasOutLog}
set tempFileList = `/bin/ls P${OBSID_XMM}*hist.qdp \
	|& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${SasOutLog}`
if ("${tempFileList}"x != x) then
    if (-e temp.qdp) /bin/rm -f temp.qdp
    foreach ii ( ${tempFileList} )
	cp -f ${ii} temp.qdp
	echo "cpd /cps" >> temp.qdp
	echo "p" >> temp.qdp
	echo "cpd /gif" >> temp.qdp
	echo "p" >> temp.qdp
	echo "cpd /xw" >> temp.qdp
	echo "p" >> temp.qdp
	echo "q" >> temp.qdp
	qdp temp.qdp
	/bin/mv -f pgplot.ps `echo ${ii} | sed "s/qdp/eps/g"`
	/bin/mv -f pgplot.gif `echo ${ii} | sed "s/qdp/gif/g"`
	/bin/rm -f temp.qdp
    end
    unset ii
endif
unset tempFileList


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (4c) <><>" |& tee -a ${SasOutLog}
#echo "<><>             define region filters; take snapshots     <><>" |& tee -a ${SasOutLog}
#echo "<><>             of images with region filters overlaid    <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...defining region filters... <><><><>" |& tee -a ${SasOutLog}
#############
# (4c) Next, ensure that region files are available and filters are defined
# for all exposures, then save snapshots with these overlaid...

### Notes on defining the source and background regions:
###
### Rather than running source detection scripts, I extract source positions
### from the region files provided by PPS (preferentially using the
### *EPX000REGION0000.ASC file as (1) it should be most comprehensive because it
### has all sources detected in joint fits of all EPIC detectors and (2) it
### provides radii that scale with the source brightness, meaning that small
### cut outs may be used when background sources are weak.
### 
### My default source and background regions differ from those of PPS in that
### (1) I use larger radii for the source regions, (2) I exclude cut-outs around
### background sources that fall within the source extraction region, and (3) my
### background source cutouts generally use smaller radii than the 1' used by
### PPS universally.
###
### The following region files will be saved (one for each instrument/exposure):
### P${OBSID_XMM}[INST][EXPID]SR_CIR0${SRCNUM_XMM}.reg = circular source region
### P${OBSID_XMM}[INST][EXPID]SR_ANN0${SRCNUM_XMM}.reg = annular source reg (w/o central 12")
### P${OBSID_XMM}[INST][EXPID]BG_ANN0${SRCNUM_XMM}.reg = source background annulus
###    (potentially contaminated by background sources)
### P${OBSID_XMM}EPX000BG_OBJ0000.reg or P${OBSID_XMM}[INST][EXPID]BG_OBJ0${SRCNUM_XMM}.reg
###    = all field sources, for removal from background (*EPX000* if a global EPIC
###      source list is used to define region file, which is preferred behavior)

# This region filter is harmless; to be used as an initialization setting...
#
setenv DummyRegFilt "!((X,Y) IN circle(999999.0,999999.0,1.0))"

# When displaying image, use minmax scaling if object is brightest in field;
# otherwise use 99.5% scaling
#
if ( ${SRCNUM_XMM} == 001 ) then
    setenv ds9scaleMode "-scale mode minmax"
else
    setenv ds9scaleMode "-scale mode 99.5"
endif

set PN_SrcCirRegFilt = ( $PNimage8list )
set PN_SrcAnnRegFilt = ( $PNimage8list )
set PN_BgdAnnRegFilt = ( $PNimage8list )
set PN_BgdObjRegFilt = ( $PNimage8list )
set PN_SrcCirRegFile = ( $PNimage8list )
set PN_SrcAnnRegFile = ( $PNimage8list )
set PN_BgdAnnRegFile = ( $PNimage8list )
set PN_BgdObjRegFile = ( $PNimage8list )

echo " "
set tmpCounter = 1
while ($tmpCounter <= $viablePNdata)
    if ((-e ${PNimage8list[$tmpCounter]}) || (-e ${PNimage1list[$tmpCounter]})) then
	# If PN image in either band exists, proceed with defining regions...
	if (-e ${PNimage8list[$tmpCounter]}) then
	    setenv tempImage ${PNimage8list[$tmpCounter]}
	else
	    setenv tempImage ${PNimage1list[$tmpCounter]}
	endif
	#
	source ${ScriptPath_XMM}buildRegFilt.csh $tempImage $SDSS_RA $SDSS_DEC $SRCNUM_XMM |& tee -a ${SasOutLog}
	#
	# To override the radii used in the extraction regions, include these
	# as the 5th-8th parameters in the argument, as in this example (which
	# shows the default values):
	# source ${ScriptPath_XMM}buildRegFilt.csh $tempImage $SDSS_RA $SDSS_DEC $SRCNUM_XMM 12 45 60 120
	# these numbers are, in order, the radius of the cut out when excluding
	# the core of the PSF, the source extraction radius, and the inner and
	# outer radii of the background annulus.
	#
	unsetenv tempImage
	#
	if (${?SrcCirFileOut}) then
	    set PN_SrcCirRegFile[$tmpCounter] = "${SrcCirFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}PN${PN_EXPID[$tmpCounter]}SR_CIR0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv SrcCirFileOut ${tempRegList[1]}
		set PN_SrcCirRegFile[$tmpCounter] = "${SrcCirFileOut}"
	    else
		echo "*** Warning: Circular source extraction region file for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcAnnFileOut}) then
	    set PN_SrcAnnRegFile[$tmpCounter]  = "${SrcAnnFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}PN${PN_EXPID[$tmpCounter]}SR_ANN0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv SrcAnnFileOut ${tempRegList[1]}
		set PN_SrcAnnRegFile[$tmpCounter] = "${SrcAnnFileOut}"
	    else
		echo "*** Warning: Annular source extraction region file for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdAnnFileOut}) then
	    set PN_BgdAnnRegFile[$tmpCounter]  = "${BgdAnnFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}PN${PN_EXPID[$tmpCounter]}BG_ANN0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv BgdAnnFileOut ${tempRegList[1]}
		set PN_BgdAnnRegFile[$tmpCounter]  = "${BgdAnnFileOut}"
	    else
		echo "*** Warning: Annular background extraction reg file for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdObjFileOut}) then
	    set PN_BgdObjRegFile[$tmpCounter]  = "${BgdObjFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}PN${PN_EXPID[$tmpCounter]}BG_OBJ0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv BgdObjFileOut ${tempRegList[1]}
		set PN_BgdObjRegFile[$tmpCounter]  = "${BgdObjFileOut}"
	    else
		echo "*** Warning: Background source selection reg file for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcCirRegStr}) then
	    set PN_SrcCirRegFilt[$tmpCounter] = "${SrcCirRegStr}"
	else
	    if (${?SrcCirFileOut}) then
		setenv SrcCirRegStr `cat ${SrcCirFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set PN_SrcCirRegFilt[$tmpCounter] = "${SrcCirRegStr}"
	    else
		set PN_SrcCirRegFilt[$tmpCounter] = "${DummyRegFilt}"
		echo "*** Warning: Circular source extraction region filter for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcAnnRegStr}) then
	    set PN_SrcAnnRegFilt[$tmpCounter]  = "${SrcAnnRegStr}"
	else
	    if (${?SrcAnnFileOut}) then
		setenv SrcAnnRegStr `cat ${SrcAnnFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set PN_SrcAnnRegFilt[$tmpCounter] = "${SrcAnnRegStr}"
	    else
		set PN_SrcAnnRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Annular source extraction region filter for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdAnnRegStr}) then
	    set PN_BgdAnnRegFilt[$tmpCounter]  = "${BgdAnnRegStr}"
	else
	    if (${?BgdAnnFileOut}) then
		setenv BgdAnnRegStr `cat ${BgdAnnFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set PN_BgdAnnRegFilt[$tmpCounter] = "${BgdAnnRegStr}"
	    else
		set PN_BgdAnnRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Annular background extraction reg filter for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdObjRegStr}) then
	    set PN_BgdObjRegFilt[$tmpCounter]  = "${BgdObjRegStr}"
	else
	    if (${?BgdObjFileOut}) then
		setenv BgdObjRegStr `cat ${BgdObjFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set PN_BgdObjRegFilt[$tmpCounter] = "${BgdObjRegStr}"
	    else
		set PN_BgdObjRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Background source selection reg filter for PN ${PN_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif

	if (-e ${PNimage8list[$tmpCounter]}) then
	    # Set up output full-band image file names, then generate snapshots:
	    #
	    setenv wideFieldJpg `echo ${PNimage8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	    setenv wideFieldEps `echo ${PNimage8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	    setenv zoomFieldJpg `echo ${PNimage8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	    setenv zoomFieldEps `echo ${PNimage8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	    #
#	    echo "Using regions $BgdAnnFileOut $BgdObjFileOut in the command"
	    ds9 -geometry 640x835 ${PNimage8list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 2 -regions $BgdAnnFileOut -regions $BgdObjFileOut -saveimage jpeg $wideFieldJpg -print destination file -print resolution 75 -print filename $wideFieldEps -cmap invert yes -print -quit
#	    echo "Using regions $SrcCirFileOut $BgdAnnFileOut in the command"
	    ds9 -geometry 640x835 ${PNimage8list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 4 -regions $SrcCirFileOut -regions $BgdAnnFileOut -saveimage jpeg $zoomFieldJpg -print destination file -print resolution 75 -print filename $zoomFieldEps -cmap invert yes -print -quit
	else
	    echo "*** Error: Full-band PN image ${PNimage8list[$tmpCounter]} not found." |& tee -a ${SasOutLog}
	endif
	if (-e ${PNimage1list[$tmpCounter]}) then
	    # Set up output full-band image file names, then generate snapshots:
	    #
	    setenv wideFieldJpg `echo ${PNimage1list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	    setenv wideFieldEps `echo ${PNimage1list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	    setenv zoomFieldJpg `echo ${PNimage1list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	    setenv zoomFieldEps `echo ${PNimage1list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	    #
#	    echo "Using regions $BgdAnnFileOut $BgdObjFileOut"
	    ds9 -geometry 640x835 ${PNimage1list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 2 -regions $BgdAnnFileOut -regions $BgdObjFileOut -saveimage jpeg $wideFieldJpg -print destination file -print resolution 75 -print filename $wideFieldEps -cmap invert yes -print -quit
#	    echo "Using regions $SrcCirFileOut $BgdAnnFileOut"
	    ds9 -geometry 640x835 ${PNimage1list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 4 -regions $SrcCirFileOut -regions $BgdAnnFileOut -saveimage jpeg $zoomFieldJpg -print destination file -print resolution 75 -print filename $zoomFieldEps -cmap invert yes -print -quit
	else
	    echo "*** Error: 200-500 eV PN image ${PNimage1list[$tmpCounter]} not found." |& tee -a ${SasOutLog}
	endif
    endif
    @ tmpCounter++
end

# Repeat this processing for the MOS detectors...

set M1_SrcCirRegFilt = ( $M1image8list )
set M1_SrcAnnRegFilt = ( $M1image8list )
set M1_BgdAnnRegFilt = ( $M1image8list )
set M1_BgdObjRegFilt = ( $M1image8list )
set M1_SrcCirRegFile = ( $M1image8list )
set M1_SrcAnnRegFile = ( $M1image8list )
set M1_BgdAnnRegFile = ( $M1image8list )
set M1_BgdObjRegFile = ( $M1image8list )

set tmpCounter = 1
while ($tmpCounter <= $viableM1data)
    if (-e ${M1image8list[$tmpCounter]}) then
	# If M1 image exists, proceed with defining regions...
	setenv tempImage ${M1image8list[$tmpCounter]}
	#
	source ${ScriptPath_XMM}buildRegFilt.csh $tempImage $SDSS_RA $SDSS_DEC $SRCNUM_XMM |& tee -a ${SasOutLog}
	#
	# To override the radii used in the extraction regions, include these
	# as the 5th-8th parameters in the argument, as in this example (which
	# shows the default values):
	# source ${ScriptPath_XMM}buildRegFilt.csh $tempImage $SDSS_RA $SDSS_DEC $SRCNUM_XMM 12 45 60 120
	# these numbers are, in order, the radius of the cut out when excluding
	# the core of the PSF, the source extraction radius, and the inner and
	# outer radii of the background annulus.
	#
	unsetenv tempImage
	#
	if (${?SrcCirFileOut}) then
	    set M1_SrcCirRegFile[$tmpCounter] = "${SrcCirFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M1${M1_EXPID[$tmpCounter]}SR_CIR0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv SrcCirFileOut ${tempRegList[1]}
		set M1_SrcCirRegFile[$tmpCounter] = "${SrcCirFileOut}"
	    else
		echo "*** Warning: Circular source extraction region file for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcAnnFileOut}) then
	    set M1_SrcAnnRegFile[$tmpCounter] = "${SrcAnnFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M1${M1_EXPID[$tmpCounter]}SR_ANN0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv SrcAnnFileOut ${tempRegList[1]}
		set M1_SrcAnnRegFile[$tmpCounter] = "${SrcAnnFileOut}"
	    else
		echo "*** Warning: Annular source extraction region file for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdAnnFileOut}) then
	    set M1_BgdAnnRegFile[$tmpCounter] = "${BgdAnnFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M1${M1_EXPID[$tmpCounter]}BG_ANN0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv BgdAnnFileOut ${tempRegList[1]}
		set M1_BgdAnnRegFile[$tmpCounter] = "${BgdAnnFileOut}"
	    else
		echo "*** Warning: Annular background extraction reg file for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdObjFileOut}) then
	    set M1_BgdObjRegFile[$tmpCounter]  = "${BgdObjFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M1${M1_EXPID[$tmpCounter]}BG_OBJ0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv BgdObjFileOut ${tempRegList[1]}
		set M1_BgdObjRegFile[$tmpCounter]  = "${BgdObjFileOut}"
	    else
		echo "*** Warning: Background source selection reg file for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcCirRegStr}) then
	    set M1_SrcCirRegFilt[$tmpCounter] = "${SrcCirRegStr}"
	else
	    if (${?SrcCirFileOut}) then
		setenv SrcCirRegStr `cat ${SrcCirFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M1_SrcCirRegFilt[$tmpCounter] = "${SrcCirRegStr}"
	    else
		set M1_SrcCirRegFilt[$tmpCounter] = "${DummyRegFilt}"
		echo "*** Warning: Circular source extraction region filter for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcAnnRegStr}) then
	    set M1_SrcAnnRegFilt[$tmpCounter]  = "${SrcAnnRegStr}"
	else
	    if (${?SrcAnnFileOut}) then
		setenv SrcAnnRegStr `cat ${SrcAnnFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M1_SrcAnnRegFilt[$tmpCounter] = "${SrcAnnRegStr}"
	    else
		set M1_SrcAnnRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Annular source extraction region filter for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdAnnRegStr}) then
	    set M1_BgdAnnRegFilt[$tmpCounter]  = "${BgdAnnRegStr}"
	else
	    if (${?BgdAnnFileOut}) then
		setenv BgdAnnRegStr `cat ${BgdAnnFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M1_BgdAnnRegFilt[$tmpCounter] = "${BgdAnnRegStr}"
	    else
		set M1_BgdAnnRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Annular background extraction reg filter for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdObjRegStr}) then
	    set M1_BgdObjRegFilt[$tmpCounter]  = "${BgdObjRegStr}"
	else
	    if (${?BgdObjFileOut}) then
		setenv BgdObjRegStr `cat ${BgdObjFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M1_BgdObjRegFilt[$tmpCounter] = "${BgdObjRegStr}"
	    else
		set M1_BgdObjRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Background source selection reg filter for M1 ${M1_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif

	# Set up output full-band image file names, then generate snapshots:
	#
	setenv wideFieldJpg `echo ${M1image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	setenv wideFieldEps `echo ${M1image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	setenv zoomFieldJpg `echo ${M1image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	setenv zoomFieldEps `echo ${M1image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	#
	ds9 -geometry 640x835 ${M1image8list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 2 -regions $BgdAnnFileOut -regions $BgdObjFileOut -saveimage jpeg $wideFieldJpg -print destination file -print resolution 75 -print filename $wideFieldEps -cmap invert yes -print -quit
	ds9 -geometry 640x835 ${M1image8list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 4 -regions $SrcCirFileOut -regions $BgdAnnFileOut -saveimage jpeg $zoomFieldJpg -print destination file -print resolution 75 -print filename $zoomFieldEps -cmap invert yes -print -quit
    else
	echo "*** Error: Full-band M1 image ${M1image8list[$tmpCounter]} not found." |& tee -a ${SasOutLog}
    endif
    @ tmpCounter++
end

set M2_SrcCirRegFilt = ( $M2image8list )
set M2_SrcAnnRegFilt = ( $M2image8list )
set M2_BgdAnnRegFilt = ( $M2image8list )
set M2_BgdObjRegFilt = ( $M2image8list )
set M2_SrcCirRegFile = ( $M2image8list )
set M2_SrcAnnRegFile = ( $M2image8list )
set M2_BgdAnnRegFile = ( $M2image8list )
set M2_BgdObjRegFile = ( $M2image8list )

set tmpCounter = 1
while ($tmpCounter <= $viableM2data)
    if (-e ${M2image8list[$tmpCounter]}) then
	# If M2 image exists, proceed with defining regions...
	setenv tempImage ${M2image8list[$tmpCounter]}
	#
	source ${ScriptPath_XMM}buildRegFilt.csh $tempImage $SDSS_RA $SDSS_DEC $SRCNUM_XMM |& tee -a ${SasOutLog}
	#
	# To override the radii used in the extraction regions, include these
	# as the 5th-8th parameters in the argument, as in this example (which
	# shows the default values):
	# source ${ScriptPath_XMM}buildRegFilt.csh $tempImage $SDSS_RA $SDSS_DEC $SRCNUM_XMM 12 45 60 120
	# these numbers are, in order, the radius of the cut out when excluding
	# the core of the PSF, the source extraction radius, and the inner and
	# outer radii of the background annulus.
	#
	unsetenv tempImage
	#
	if (${?SrcCirFileOut}) then
	    set M2_SrcCirRegFile[$tmpCounter] = "${SrcCirFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M2${M2_EXPID[$tmpCounter]}SR_CIR0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv SrcCirFileOut ${tempRegList[1]}
		set M2_SrcCirRegFile[$tmpCounter] = "${SrcCirFileOut}"
	    else
		echo "*** Warning: Circular source extraction region file for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcAnnFileOut}) then
	    set M2_SrcAnnRegFile[$tmpCounter]  = "${SrcAnnFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M2${M2_EXPID[$tmpCounter]}SR_ANN0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv SrcAnnFileOut ${tempRegList[1]}
		set M2_SrcAnnRegFile[$tmpCounter] = "${SrcAnnFileOut}"
	    else
		echo "*** Warning: Annular source extraction region file for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdAnnFileOut}) then
	    set M2_BgdAnnRegFile[$tmpCounter]  = "${BgdAnnFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M2${M2_EXPID[$tmpCounter]}BG_ANN0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv BgdAnnFileOut ${tempRegList[1]}
		set M2_BgdAnnRegFile[$tmpCounter]  = "${BgdAnnFileOut}"
	    else
		echo "*** Warning: Annular background extraction reg file for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdObjFileOut}) then
	    set M2_BgdObjRegFile[$tmpCounter]  = "${BgdObjFileOut}"
	else
	    set tempRegList = `/bin/ls -t P${OBSID_XMM}M2${M2_EXPID[$tmpCounter]}BG_OBJ0${SRCNUM_XMM}.reg`
	    if ("${tempRegList}"x != x) then
		setenv BgdObjFileOut ${tempRegList[1]}
		set M2_BgdObjRegFile[$tmpCounter]  = "${BgdObjFileOut}"
	    else
		echo "*** Warning: Background source selection reg file for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcCirRegStr}) then
	    set M2_SrcCirRegFilt[$tmpCounter] = "${SrcCirRegStr}"
	else
	    if (${?SrcCirFileOut}) then
		setenv SrcCirRegStr `cat ${SrcCirFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M2_SrcCirRegFilt[$tmpCounter] = "${SrcCirRegStr}"
	    else
		set M2_SrcCirRegFilt[$tmpCounter] = "${DummyRegFilt}"
		echo "*** Warning: Circular source extraction region filter for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?SrcAnnRegStr}) then
	    set M2_SrcAnnRegFilt[$tmpCounter]  = "${SrcAnnRegStr}"
	else
	    if (${?SrcAnnFileOut}) then
		setenv SrcAnnRegStr `cat ${SrcAnnFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M2_SrcAnnRegFilt[$tmpCounter] = "${SrcAnnRegStr}"
	    else
		set M2_SrcAnnRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Annular source extraction region filter for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdAnnRegStr}) then
	    set M2_BgdAnnRegFilt[$tmpCounter]  = "${BgdAnnRegStr}"
	else
	    if (${?BgdAnnFileOut}) then
		setenv BgdAnnRegStr `cat ${BgdAnnFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M2_BgdAnnRegFilt[$tmpCounter] = "${BgdAnnRegStr}"
	    else
		set M2_BgdAnnRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Annular background extraction reg filter for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif
	if (${?BgdObjRegStr}) then
	    set M2_BgdObjRegFilt[$tmpCounter]  = "${BgdObjRegStr}"
	else
	    if (${?BgdObjFileOut}) then
		setenv BgdObjRegStr `cat ${BgdObjFileOut} | grep SASfilter | sed "s/# SASfilter: //g"`
		set M2_BgdObjRegFilt[$tmpCounter] = "${BgdObjRegStr}"
	    else
		set M2_BgdObjRegFilt[$tmpCounter]  = "${DummyRegFilt}"
		echo "*** Warning: Background source selection reg filter for M2 ${M2_EXPID[$tmpCounter]} not defined." |& tee -a ${SasOutLog}
	    endif
	endif

	# Set up output full-band image file names, then generate snapshots:
	#
	setenv wideFieldJpg `echo ${M2image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	setenv wideFieldEps `echo ${M2image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_BGD/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	setenv zoomFieldJpg `echo ${M2image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/jpg/g' -e "s/000/${SRCNUM_XMM}/g"`
	setenv zoomFieldEps `echo ${M2image8list[$tmpCounter]} | sed -e 's/IMAGE_/IM_SRC/g' -e 's/FTZ/FIT/g' -e 's/FIT/eps/g' -e "s/000/${SRCNUM_XMM}/g"`
	#
	ds9 -geometry 640x835 ${M2image8list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 2 -regions $BgdAnnFileOut -regions $BgdObjFileOut -saveimage jpeg $wideFieldJpg -print destination file -print resolution 75 -print filename $wideFieldEps -cmap invert yes -print -quit
	ds9 -geometry 640x835 ${M2image8list[$tmpCounter]} ${ds9scaleMode} -log -pan to $SDSS_RA "$SDSS_DEC" wcs fk5 -zoom to 4 -regions $SrcCirFileOut -regions $BgdAnnFileOut -saveimage jpeg $zoomFieldJpg -print destination file -print resolution 75 -print filename $zoomFieldEps -cmap invert yes -print -quit
    else
	echo "*** Error: Full-band M2 image ${M2image8list[$tmpCounter]} not found." |& tee -a ${SasOutLog}
    endif
    @ tmpCounter++
end

unset tempRegList
unsetenv DummyRegFilt
unsetenv SrcCirFileOut
unsetenv SrcAnnFileOut
unsetenv BgdAnnFileOut
unsetenv BgdObjFileOut
unsetenv wideFieldJpg
unsetenv wideFieldEps
unsetenv zoomFieldJpg
unsetenv zoomFieldEps


# Not implemented: any sort of test/checking whether region filters are
# identical to DummyRegFilt.

# Also not implemented: Test whether source is covered by instrument?  How?  Use
# Funtools to get net counts?  Just get central pixel value from PPS exposure
# map?  If no coverage, flag as not covered and do not attempt to proceed
# further with this instrument.


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (5) <><>" |& tee -a ${SasOutLog}
#echo "<><>             this section does nothing                <><>" |& tee -a ${SasOutLog}
#############
# (5) Repeat step (4) showing just the soft-band PN image with source
# regions overlaid.  Save snapshot for later inspection.
#
# Now implemented as part of (4)
#
#
# Not implemented: an automated way to identify soft background noise.
#
# One possibility: if source is sufficiently close to RawY=0, then measure the
# low-energy counts per pixel at the high- and low-RawY ends of the background
# region.  Significant difference?  Then set a PN soft background noise flag.
# 
# Another: Any time a source has too low a value of RawY, a grade 0 PN spectrum
# should be extracted so that it would be available later if the user wishes.
# (Using only pattern==0 events mitigates the soft background noise).


#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (6) <><>" |& tee -a ${SasOutLog}
#echo "<><>             this section does nothing                <><>" |& tee -a ${SasOutLog}
#############
# (6) check count rates for pileup thresholds.
#
# Compare PN_8_RATE, M1_8_RATE, and M2_8_RATE to thresholds (as
# function of observing mode) and set pileup flags accordingly. 
# Two flags: one for pileup in double events, one for pileup in
# singles.  If the latter is set, then the former is automatically
# also set.


#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (7) <><>" |& tee -a ${SasOutLog}
#echo "<><>             extract spectra                          <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...extracting spectra... <><><><>" |& tee -a ${SasOutLog}
echo " " |& tee -a ${SasOutLog}
#############
# (7) Spectra (re-)extractions
# 
# Key to spectral file naming conventions:
# SRSPEC = source spectrum taken from circular aperture and with all reasonable events
# BGSPEC = background spectrum taken from annular aperture and with all reasonable events
# SRSP_A = source spectrum taken from annular aperture and with all reasonable events
# SRSP_0 = pattern==0 source spectrum taken from a circular aperture
# BGSP_0 = pattern==0 background spectrum taken from a circular aperture
# BGSP_S = background spectrum taken from a source-free field using the circular source region
#
# [INST]SPEClist lists the *SRSPEC* spectral files for each exposure with each
# instrument (which may include PPS-generated spectra if $UsePPSspectra == YES
# and such files exist).  To this point, lists of associated and alternate
# spectra (BGSPEC, SRSP_A and SRSP_0) have not been generated.
#
# There are lists of flags (extract[INST]SPEC) to indicate which *SRSPEC* files
# need to be generated, but (1) this can be obviated by using "if (-e ...)" and
# (2) associated/alternative spectra may need extraction even when SRSPEC is
# already available.  If I don't use these flags, I should remove them from the
# code.
#
# Any other extractions to make? 
#
# Backgrounds from blank-sky composite images?  Likely will be necessary in some
# windowing modes.  By default, get this universally?  If not, how to decide
# when?  How can I determine when a source lies within a window small enough to
# require such a background?  Alternatively, I could assume this is only needed
# when in small window mode and ${SRCNUM_XMM} == 001...
#
# Do I need PATTERN==0 background spectra?  When fitting source spectra, do the
# background spectra have to use the same response matrices?  This clearly would
# be inappropriate.
#
if (${viablePNdata} > 0) then
    echo "  <><><> ...starting with the PN exposure(s)... <><><>" |& tee -a ${SasOutLog}
    set PNBGSPEClist = ( ${PNSPEClist} )
    set PNSRSP_Alist = ( ${PNSPEClist} )
    set PNSRSP_0list = ( ${PNSPEClist} )
    set PNBGSP_0list = ( ${PNSPEClist} )
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viablePNdata} )
	# SRSPEC:
	set tmpSPECname = ${PNSPEClist[$tmpCounter]}
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set tmpEVTname = ${PN_EVTlist[$tmpCounter]}
	if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${PN_SrcCirRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
	    set selectExpr = "(FLAG==0) && (PATTERN<=4) && ${PN_SrcCirRegFilt[$tmpCounter]} && gti(${PN_FlareGTI[$tmpCounter]},TIME)"
	    source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		"${selectExpr}" ${SasOutLog}
	endif
	source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
	    ${tmpEVTname} NONE ${SasOutLog}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
# echo "    <> Final BACKSCAL value is ${tempBackScal} for source ${tmpSPECname}" |& tee -a ${SasOutLog}
# echo "       BackScalTest is set to ${BackScalTest}" |& tee -a ${SasOutLog}
	# BGSPEC:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/SRSPEC/BGSPEC/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    # ...the above shouldn't be necessary because SRSPEC should already be *.FIT
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	if (! -e $tmpSPECname) set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	set PNBGSPEClist[$tmpCounter] = $tmpSPECname
	if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${PN_BgdAnnRegFilt[$tmpCounter]} && ${PN_BgdObjRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
	    set selectExpr = "(FLAG==0) && (PATTERN<=4) && ${PN_BgdAnnRegFilt[$tmpCounter]} && ${PN_BgdObjRegFilt[$tmpCounter]} && gti(${PN_FlareGTI[$tmpCounter]},TIME)"
	    source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		"${selectExpr}" ${SasOutLog}
	endif
	source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
	    ${tmpEVTname} NONE ${SasOutLog}
	# BGSP_0:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g" -e "s/BGSPEC/BGSP_0/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set PNBGSP_0list[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${PN_BgdAnnRegFilt[$tmpCounter]} && ${PN_BgdObjRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "(FLAG==0) && (PATTERN==0) && ${PN_BgdAnnRegFilt[$tmpCounter]} && ${PN_BgdObjRegFilt[$tmpCounter]} && gti(${PN_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} ${PNBGSPEClist[$tmpCounter]} ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	# SRSP_0:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/BGSP_0/SRSP_0/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set PNSRSP_0list[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${PN_SrcCirRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "(FLAG==0) && (PATTERN==0) && ${PN_SrcCirRegFilt[$tmpCounter]} && gti(${PN_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} ${PNSPEClist[$tmpCounter]} ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	# SRSP_A:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/SRSP_0/SRSP_A/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set PNSRSP_Alist[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${PN_SrcAnnRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "(FLAG==0) && (PATTERN<=4) && ${PN_SrcAnnRegFilt[$tmpCounter]} && gti(${PN_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} NONE ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	@ tmpCounter++
    end
endif

#echo "<><><>    Debug test 1: done generating PN spectra    <><><>"
# echo "  <><> ...finished extracting PN spectra... <><>"

# ...and now for the MOS spectra:

if (${viableM1data} > 0) then
    echo "  <><><> ...continuing with the MOS1 exposure(s)... <><><>" |& tee -a ${SasOutLog}
    set M1BGSPEClist = ( ${M1SPEClist} )
    set M1SRSP_Alist = ( ${M1SPEClist} )
    set M1SRSP_0list = ( ${M1SPEClist} )
    set M1BGSP_0list = ( ${M1SPEClist} )
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viableM1data} )
	# SRSPEC:
	set tmpSPECname = ${M1SPEClist[$tmpCounter]}
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set tmpEVTname = ${M1_EVTlist[$tmpCounter]}
	if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M1_SrcCirRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
	    set selectExpr = "#XMMEA_SM && (PATTERN<=12) && ${M1_SrcCirRegFilt[$tmpCounter]} && gti(${M1_FlareGTI[$tmpCounter]},TIME)"
	    source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		"${selectExpr}" ${SasOutLog}
	endif
	source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
	    ${tmpEVTname} NONE ${SasOutLog}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
# echo "    <> Final BACKSCAL value is ${tempBackScal} for source ${tmpSPECname}" |& tee -a ${SasOutLog}
# echo "       BackScalTest is set to ${BackScalTest}" |& tee -a ${SasOutLog}
	# BGSPEC:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/SRSPEC/BGSPEC/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    # ...the above shouldn't be necessary because SRSPEC should already be *.FIT
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	if (! -e $tmpSPECname) set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	set M1BGSPEClist[$tmpCounter] = $tmpSPECname
	if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M1_BgdAnnRegFilt[$tmpCounter]} && ${M1_BgdObjRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
	    set selectExpr = "#XMMEA_SM && (PATTERN<=12) && ${M1_BgdAnnRegFilt[$tmpCounter]} && ${M1_BgdObjRegFilt[$tmpCounter]} && gti(${M1_FlareGTI[$tmpCounter]},TIME)"
	    source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		"${selectExpr}" ${SasOutLog}
	endif
	source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
	    ${tmpEVTname} NONE ${SasOutLog}
	# BGSP_0:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g" -e "s/BGSPEC/BGSP_0/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set M1BGSP_0list[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M1_BgdAnnRegFilt[$tmpCounter]} && ${M1_BgdObjRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "#XMMEA_SM && (PATTERN==0) && ${M1_BgdAnnRegFilt[$tmpCounter]} && ${M1_BgdObjRegFilt[$tmpCounter]} && gti(${M1_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} ${M1BGSPEClist[$tmpCounter]} ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	# SRSP_0:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/BGSP_0/SRSP_0/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set M1SRSP_0list[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M1_SrcCirRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "#XMMEA_SM && (PATTERN==0) && ${M1_SrcCirRegFilt[$tmpCounter]} && gti(${M1_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} ${M1SPEClist[$tmpCounter]} ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	# SRSP_A:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/SRSP_0/SRSP_A/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set M1SRSP_Alist[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M1_SrcAnnRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "#XMMEA_SM && (PATTERN<=12) && ${M1_SrcAnnRegFilt[$tmpCounter]} && gti(${M1_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} NONE ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	@ tmpCounter++
    end
endif

# echo "<><><>    Debug test 2: done generating MOS1 spectra    <><><>"
# echo "  <><> ...finished extracting MOS1 spectra... <><>"

if (${viableM2data} > 0) then
    echo "  <><><> ...and finally the MOS2 exposure(s)... <><><>" |& tee -a ${SasOutLog}
    set M2BGSPEClist = ( ${M2SPEClist} )
    set M2SRSP_Alist = ( ${M2SPEClist} )
    set M2SRSP_0list = ( ${M2SPEClist} )
    set M2BGSP_0list = ( ${M2SPEClist} )
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viableM2data} )
	# SRSPEC:
	set tmpSPECname = ${M2SPEClist[$tmpCounter]}
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set tmpEVTname = ${M2_EVTlist[$tmpCounter]}
	if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M2_SrcCirRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
	    set selectExpr = "#XMMEA_SM && (PATTERN<=12) && ${M2_SrcCirRegFilt[$tmpCounter]} && gti(${M2_FlareGTI[$tmpCounter]},TIME)"
	    source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		"${selectExpr}" ${SasOutLog}
	endif
	source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
	    ${tmpEVTname} NONE ${SasOutLog}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
# echo "    <> Final BACKSCAL value is ${tempBackScal} for source ${tmpSPECname}" |& tee -a ${SasOutLog}
# echo "       BackScalTest is set to ${BackScalTest}" |& tee -a ${SasOutLog}
	# BGSPEC:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/SRSPEC/BGSPEC/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    # ...the above shouldn't be necessary because SRSPEC should already be *.FIT
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	if (! -e $tmpSPECname) set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	set M2BGSPEClist[$tmpCounter] = $tmpSPECname
	if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M2_BgdAnnRegFilt[$tmpCounter]} && ${M2_BgdObjRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
	    set selectExpr = "#XMMEA_SM && (PATTERN<=12) && ${M2_BgdAnnRegFilt[$tmpCounter]} && ${M2_BgdObjRegFilt[$tmpCounter]} && gti(${M2_FlareGTI[$tmpCounter]},TIME)"
	    source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		"${selectExpr}" ${SasOutLog}
	endif
	source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
	    ${tmpEVTname} NONE ${SasOutLog}
	# BGSP_0:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g" -e "s/BGSPEC/BGSP_0/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set M2BGSP_0list[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M2_BgdAnnRegFilt[$tmpCounter]} && ${M2_BgdObjRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "#XMMEA_SM && (PATTERN==0) && ${M2_BgdAnnRegFilt[$tmpCounter]} && ${M2_BgdObjRegFilt[$tmpCounter]} && gti(${M2_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} ${M2BGSPEClist[$tmpCounter]} ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	# SRSP_0:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/BGSP_0/SRSP_0/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set M2SRSP_0list[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M2_SrcCirRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "#XMMEA_SM && (PATTERN==0) && ${M2_SrcCirRegFilt[$tmpCounter]} && gti(${M2_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} ${M2SPEClist[$tmpCounter]} ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	# SRSP_A:
	set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/SRSP_0/SRSP_A/g"`
	if ( ${reExtractSpec} == YES ) then
	    set tmpSPECname = `echo ${tmpSPECname} | sed -e "s/FTZ/FIT/g"`
	    if (-e $tmpSPECname) /bin/rm -f ${tmpSPECname}
	endif
	set M2SRSP_Alist[$tmpCounter] = $tmpSPECname
	if ("${BackScalTest}" != "-") then
	    # ie., if the source spectrum is drawn from a reasonable amount of area...
	    if (! -e $tmpSPECname) then
echo "> <> < Extracting ${tmpSPECname}..." |& tee -a ${SasOutLog}
echo "       using the region filter ${M2_SrcAnnRegFilt[$tmpCounter]}" |& tee -a ${SasOutLog}
		set selectExpr = "#XMMEA_SM && (PATTERN<=12) && ${M2_SrcAnnRegFilt[$tmpCounter]} && gti(${M2_FlareGTI[$tmpCounter]},TIME)"
		source ${ScriptPath_XMM}specExtract.csh ${tmpEVTname} ${tmpSPECname} \
		    "${selectExpr}" ${SasOutLog}
	    endif
	    source ${ScriptPath_XMM}backscalReset.csh ${tmpSPECname} ${minSrcArea} \
		${tmpEVTname} NONE ${SasOutLog}
	else
	    echo "     ${tmpSPECname} not extracted because the good detector area" |& tee -a ${SasOutLog}
	    echo "       covered by the corresponding circular source aperture is only `calc ${tempBackScal} / 400.0` square arcsec" |& tee -a ${SasOutLog}
	endif
	@ tmpCounter++
    end
endif

# echo "<><><>    Debug test 3: done generating MOS2 spectra    <><><>"
# echo "  <><> ...finished extracting MOS2 spectra... <><>"

if (${?tmpEVTname}) unset tmpEVTname
if (${?tmpSPECname}) unset tmpSPECname
if (${?BackScalTest}) unset BackScalTest
if (${?BackScalTest2}) unset BackScalTest2
if (${?tempBackScal}) unset tempBackScal
if (${?tempBackScal2}) unset tempBackScal2

# No automatic triggers for extraction field contamination (4)?
#
# PN soft background noise (5):
# Automatically extract a pattern == 0 PN spectrum if RawY value is too low.
# Rely upon PN soft background noise flag to determine whether this will be used
# in the pipeline or just readied for the user?


# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (8) <><>" |& tee -a ${SasOutLog}
# echo "<><>             this section does nothing                <><>" |& tee -a ${SasOutLog}
#############
# (8) Define extraction regions.
#
# This is now done as part of step (4c).


# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (9) <><>" |& tee -a ${SasOutLog}
# echo "<><>             this section does nothing                <><>" |& tee -a ${SasOutLog}
#############
# (9) Determine flare background GTIs.
#
# This is now done within section (4b) using the "bgdFlareGTI.csh" script.


#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (10) <><>" |& tee -a ${SasOutLog}
#echo "<><>             this section does nothing                 <><>" |& tee -a ${SasOutLog}
#############
# (10) (Re-)extract spectra
#
# ...only if necessary/useful.


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (10a) <><>" |& tee -a ${SasOutLog}
#echo "<><>             generating diagnostic figures              <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...generating diagnostic figures... <><><><>" |& tee -a ${SasOutLog}
#############
# (10a) Generate diagnostic figures
#
# epatplot?
# ratios of all-event, singles-only, and annulus-only spectra?

echo " " |& tee -a ${SasOutLog}
echo "Now generating pattern plots for diagnosing event pile-up:" |& tee -a ${SasOutLog}

if (${viablePNdata} > 0) then
    set tmpCounter = 1
    foreach ii ( ${PN_EVTlist} )
	set tmpSPECname = ${PNSPEClist[$tmpCounter]}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	if ("${BackScalTest}" != "-") then 
	    set selectExpr = "(FLAG==0) && gti(${PN_FlareGTI[$tmpCounter]},TIME) && ${PN_SrcCirRegFilt[$tmpCounter]}"
	    source ${ScriptPath_XMM}runEpatplot.csh $ii ${SRCNUM_XMM} "${selectExpr}" ${SasOutLog}
	endif
	@ tmpCounter++
    end
    unset ii
endif

if (${viableM1data} > 0) then
    set tmpCounter = 1
    foreach ii ( ${M1_EVTlist} )
	set tmpSPECname = ${M1SPEClist[$tmpCounter]}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	if ("${BackScalTest}" != "-") then 
	    set selectExpr = "(FLAG==0) && gti(${M1_FlareGTI[$tmpCounter]},TIME) && ${M1_SrcCirRegFilt[$tmpCounter]}"
	    source ${ScriptPath_XMM}runEpatplot.csh $ii ${SRCNUM_XMM} "${selectExpr}" ${SasOutLog}
	endif
	@ tmpCounter++
    end
    unset ii
endif

if (${viableM2data} > 0) then
    set tmpCounter = 1
    foreach ii ( ${M2_EVTlist} )
	set tmpSPECname = ${M2SPEClist[$tmpCounter]}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	if ("${BackScalTest}" != "-") then 
	    set selectExpr = "(FLAG==0) && gti(${M2_FlareGTI[$tmpCounter]},TIME) && ${M2_SrcCirRegFilt[$tmpCounter]}"
	    source ${ScriptPath_XMM}runEpatplot.csh $ii ${SRCNUM_XMM} "${selectExpr}" ${SasOutLog}
	endif
	@ tmpCounter++
    end
    unset ii
endif

if (${?selectExpr}) unset selectExpr
if (${?tmpSPECname}) unset tmpSPECname
if (${?tempBackScal}) unset tempBackScal
if (${?BackScalTest}) unset BackScalTest


#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (11) <><>" |& tee -a ${SasOutLog}
#echo "<><>             generating RMF and ARF matrices           <><>" |& tee -a ${SasOutLog}
echo " " |& tee -a ${SasOutLog}
echo "<><><><> ...generating RMF and ARF matrices... <><><><>" |& tee -a ${SasOutLog}
#############
# (11) Generate response matrices
#
set PNRMF_list = ( $PNSPEClist )
set PNRMF0list = ( $PNSPEClist )
set PNARF_list = ( $PNSPEClist )
set PNARF0list = ( $PNSPEClist )
set PNARFAlist = ( $PNSPEClist )

if (${viablePNdata} > 0) then
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viablePNdata} )
	set tmpSPECname = ${PNSPEClist[$tmpCounter]}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	if ("${BackScalTest}" != "-") then 
	    set tmpSPECname = `echo ${tmpSPECname} | sed 's/FTZ/FIT/g'`
	    set PNRMF_list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRRMF_/g'`
	    set PNRMF0list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRRMF0/g'`
	    set PNARF_list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARF_/g'`
	    set PNARF0list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARF0/g'`
	    set PNARFAlist[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARFA/g'`
# echo "Debug test  1 - PN SRRMF_ - RMF for all good event patterns"
echo "> <> < SRRMF_ for PN (${PN_EXPID[$tmpCounter]}): RMF for all good event patterns" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${PNRMF_list[$tmpCounter]}
	    if (! -e ${PNRMF_list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		rmfgen rmfset=${PNRMF_list[$tmpCounter]} \
		    spectrumset=${PNSPEClist[$tmpCounter]} detmaptype=psf >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this RMF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  2 - PN SRRMF0 - RMF for events with pattern == 0"
echo "> <> < SRRMF0 for PN (${PN_EXPID[$tmpCounter]}): RMF for all events with pattern == 0" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${PNRMF0list[$tmpCounter]}
	    if (! -e ${PNRMF0list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		rmfgen rmfset=${PNRMF0list[$tmpCounter]} \
		    spectrumset=${PNSRSP_0list[$tmpCounter]} detmaptype=psf >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this RMF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  3 - PN SRARF_ - ARF for all good event patterns in circular src reg"
echo "> <> < SRARF_ for PN (${PN_EXPID[$tmpCounter]}): ARF for all good event patterns in circular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${PNARF_list[$tmpCounter]}
	    if (! -e ${PNARF_list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${PNARF_list[$tmpCounter]} setbackscale=yes \
		    spectrumset=${PNSPEClist[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${PN_EVTlist[$tmpCounter]} \
		    rmfset=${PNRMF_list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  4 - PN SRARF0 - ARF for all pattern==0 events in circular src reg"
echo "> <> < SRARF0 for PN (${PN_EXPID[$tmpCounter]}): ARF for all pattern==0 events in circular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${PNARF0list[$tmpCounter]}
	    if (! -e ${PNARF0list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${PNARF0list[$tmpCounter]} setbackscale=yes \
		    spectrumset=${PNSRSP_0list[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${PN_EVTlist[$tmpCounter]} \
		    rmfset=${PNRMF0list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  5 - PN SRARFA - ARF for all good event patterns in annular src reg"
echo "> <> < SRARFA for PN (${PN_EXPID[$tmpCounter]}): ARF for all good event patterns in annular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${PNARFAlist[$tmpCounter]}
	    if (! -e ${PNARFAlist[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${PNARFAlist[$tmpCounter]} setbackscale=yes \
		    spectrumset=${PNSRSP_Alist[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${PN_EVTlist[$tmpCounter]} \
		    rmfset=${PNRMF_list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
	endif
	@ tmpCounter++
    end
endif

set M1RMF_list = ( $M1SPEClist )
set M1RMF0list = ( $M1SPEClist )
set M1ARF_list = ( $M1SPEClist )
set M1ARF0list = ( $M1SPEClist )
set M1ARFAlist = ( $M1SPEClist )

if (${viableM1data} > 0) then
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viableM1data} )
	set tmpSPECname = ${M1SPEClist[$tmpCounter]}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	if ("${BackScalTest}" != "-") then 
	    set tmpSPECname = `echo ${tmpSPECname} | sed 's/FTZ/FIT/g'`
	    set M1RMF_list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRRMF_/g'`
	    set M1RMF0list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRRMF0/g'`
	    set M1ARF_list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARF_/g'`
	    set M1ARF0list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARF0/g'`
	    set M1ARFAlist[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARFA/g'`
# echo "Debug test  6 - M1 SRRMF_ - RMF for all good event patterns"
echo "> <> < SRRMF_ for M1 (${M1_EXPID[$tmpCounter]}): RMF for all good event patterns" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M1RMF_list[$tmpCounter]}
	    if (! -e ${M1RMF_list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		rmfgen rmfset=${M1RMF_list[$tmpCounter]} \
		    spectrumset=${M1SPEClist[$tmpCounter]} detmaptype=psf >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this RMF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  7 - M1 SRRMF0 - RMF for events with pattern == 0"
echo "> <> < SRRMF0 for M1 (${M1_EXPID[$tmpCounter]}): RMF for all events with pattern == 0" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M1RMF0list[$tmpCounter]}
	    if (! -e ${M1RMF0list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		rmfgen rmfset=${M1RMF0list[$tmpCounter]} \
		    spectrumset=${M1SRSP_0list[$tmpCounter]} detmaptype=psf >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this RMF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  8 - M1 SRARF_ - ARF for all good event patterns in circular src reg"
echo "> <> < SRARF_ for M1 (${M1_EXPID[$tmpCounter]}): ARF for all good event patterns in circular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M1ARF_list[$tmpCounter]}
	    if (! -e ${M1ARF_list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${M1ARF_list[$tmpCounter]} setbackscale=yes \
		    spectrumset=${M1SPEClist[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${M1_EVTlist[$tmpCounter]} \
		    rmfset=${M1RMF_list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test  9 - M1 SRARF0 - ARF for all pattern==0 events in circular src reg"
echo "> <> < SRARF0 for M1 (${M1_EXPID[$tmpCounter]}): ARF for all pattern==0 events in circular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M1ARF0list[$tmpCounter]}
	    if (! -e ${M1ARF0list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${M1ARF0list[$tmpCounter]} setbackscale=yes \
		    spectrumset=${M1SRSP_0list[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${M1_EVTlist[$tmpCounter]} \
		    rmfset=${M1RMF0list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test 10 - M1 SRARFA - ARF for all good event patterns in annular src reg"
echo "> <> < SRARFA for M1 (${M1_EXPID[$tmpCounter]}): ARF for all good event patterns in annular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M1ARFAlist[$tmpCounter]}
	    if (! -e ${M1ARFAlist[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${M1ARFAlist[$tmpCounter]} setbackscale=yes \
		    spectrumset=${M1SRSP_Alist[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${M1_EVTlist[$tmpCounter]} \
		    rmfset=${M1RMF_list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
	endif
	@ tmpCounter++
    end
endif

set M2RMF_list = ( $M2SPEClist )
set M2RMF0list = ( $M2SPEClist )
set M2ARF_list = ( $M2SPEClist )
set M2ARF0list = ( $M2SPEClist )
set M2ARFAlist = ( $M2SPEClist )

if (${viableM2data} > 0) then
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viableM2data} )
	set tmpSPECname = ${M2SPEClist[$tmpCounter]}
	fkeypar ${tmpSPECname} BACKSCAL
	set tempBackScal = `pget fkeypar value`
	set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	if ("${BackScalTest}" != "-") then 
	    set tmpSPECname = `echo ${tmpSPECname} | sed 's/FTZ/FIT/g'`
	    set M2RMF_list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRRMF_/g'`
	    set M2RMF0list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRRMF0/g'`
	    set M2ARF_list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARF_/g'`
	    set M2ARF0list[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARF0/g'`
	    set M2ARFAlist[$tmpCounter] = `echo ${tmpSPECname} | sed 's/SRSPEC/SRARFA/g'`
# echo "Debug test 11 - M2 SRRMF_ - RMF for all good event patterns"
echo "> <> < SRRMF_ for M2 (${M2_EXPID[$tmpCounter]}): RMF for all good event patterns" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M2RMF_list[$tmpCounter]}
	    if (! -e ${M2RMF_list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		rmfgen rmfset=${M2RMF_list[$tmpCounter]} \
		    spectrumset=${M2SPEClist[$tmpCounter]} detmaptype=psf >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this RMF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test 12 - M2 SRRMF0 - RMF for events with pattern == 0"
echo "> <> < SRRMF0 for M2 (${M2_EXPID[$tmpCounter]}): RMF for all events with pattern == 0" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M2RMF0list[$tmpCounter]}
	    if (! -e ${M2RMF0list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		rmfgen rmfset=${M2RMF0list[$tmpCounter]} \
		    spectrumset=${M2SRSP_0list[$tmpCounter]} detmaptype=psf >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this RMF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test 13 - M2 SRARF_ - ARF for all good event patterns in circular src reg"
echo "> <> < SRARF_ for M2 (${M2_EXPID[$tmpCounter]}): ARF for all good event patterns in circular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M2ARF_list[$tmpCounter]}
	    if (! -e ${M2ARF_list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${M2ARF_list[$tmpCounter]} setbackscale=yes \
		    spectrumset=${M2SPEClist[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${M2_EVTlist[$tmpCounter]} \
		    rmfset=${M2RMF_list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test 14 - M2 SRARF0 - ARF for all pattern==0 events in circular src reg"
echo "> <> < SRARF0 for M2 (${M2_EXPID[$tmpCounter]}): ARF for all pattern==0 events in circular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M2ARF0list[$tmpCounter]}
	    if (! -e ${M2ARF0list[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${M2ARF0list[$tmpCounter]} setbackscale=yes \
		    spectrumset=${M2SRSP_0list[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${M2_EVTlist[$tmpCounter]} \
		    rmfset=${M2RMF0list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
# echo "Debug test 15 - M2 SRARFA - ARF for all good event patterns in annular src reg"
echo "> <> < SRARFA for M2 (${M2_EXPID[$tmpCounter]}): ARF for all good event patterns in annular src reg" |& tee -a ${SasOutLog}
	    if ( ${reGenerateMatrices} == YES ) /bin/rm -f ${M2ARFAlist[$tmpCounter]}
	    if (! -e ${M2ARFAlist[$tmpCounter]}) then
		if (-e tmp.out) /bin/rm -f tmp.out
		arfgen arfset=${M2ARFAlist[$tmpCounter]} setbackscale=yes \
		    spectrumset=${M2SRSP_Alist[$tmpCounter]} detmaptype=psf \
		    badpixelresolution=1.0 withsourcepos=false withrmfset=yes \
		    extendedsource=no badpixlocation=${M2_EVTlist[$tmpCounter]} \
		    rmfset=${M2RMF_list[$tmpCounter]} modelee=yes >>&! tmp.out
		cat tmp.out >>! ${SasOutLog}
		echo "       Warnings and errors that happened when creating this ARF:" |& tee -a ${SasOutLog}
		# grep "\*\*" tmp.out |& tee -a ${SasOutLog}
		grep -i " warning " tmp.out |& tee -a ${SasOutLog}
		grep -i " error " tmp.out |& tee -a ${SasOutLog}
		/bin/rm -f tmp.out
	    endif
	endif
	@ tmpCounter++
    end
endif

if (${?tmpSPECname}) unset tmpSPECname
if (${?tempBackScal}) unset tempBackScal
if (${?BackScalTest}) unset BackScalTest


#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (12) <><>" |& tee -a ${SasOutLog}
#echo "<><>             final spectra setup with GRPPHA           <><>" |& tee -a ${SasOutLog}
echo " " |& tee -a ${SasOutLog}
echo "<><><><> ...setting up spectra with GRPPHA... <><><><>" |& tee -a ${SasOutLog}
#############
# (12) Run GRPPHA to prep the data for XSPEC
#
# File naming convention: when running GRPPHA, the output files should have
# "_grpNN" appended to their names just prior to the extension ".FIT", where NN
# is the size of the grouping.  When GRPPHA is first run to add header keywords
# and do no grouping, the extension would be "_grp00".  The second pass, when
# we use "group min 25", the extension becomes "_grp25"...
#

if (${viablePNdata} > 0) then
    if (-e tmp.out) /bin/rm -f tmp.out
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viablePNdata} )
	#
	# SRSPEC (events with all valid patterns, circular aperture):
	set tmpSPEC0 = ${PNSPEClist[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${PNBGSPEClist[$tmpCounter]}.gz"
	    set tmpRMF   = "${PNRMF_list[$tmpCounter]}.gz"
	    set tmpARF   = "${PNARF_list[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-39 1576-1646 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-39 1576-1646 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	#
	# SRSP_0 (only pattern==0 events, circular aperture):
	set tmpSPEC0 = ${PNSRSP_0list[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${PNBGSP_0list[$tmpCounter]}.gz"
	    set tmpRMF   = "${PNRMF0list[$tmpCounter]}.gz"
	    set tmpARF   = "${PNARF0list[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-39 1576-1646 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-39 1576-1646 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	#
	# SRSP_A (events with all valid patterns, annular aperture):
	set tmpSPEC0 = ${PNSRSP_Alist[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${PNBGSPEClist[$tmpCounter]}.gz"
	    set tmpRMF   = "${PNRMF_list[$tmpCounter]}.gz"
	    set tmpARF   = "${PNARFAlist[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-39 1576-1646 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-39 1576-1646 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	@ tmpCounter++
    end
    echo "Output from running GRPPHA on PN spectra:" >>! ${SasOutLog}
    cat tmp.out >>! ${SasOutLog}
    # echo "If any errors were encountered when running GRPPHA on the PN spectra," |& tee -a ${SasOutLog}
    # echo "   these appear on the following lines:" |& tee -a ${SasOutLog}
    echo "       GRPPHA warnings and errors for PN spectra:" |& tee -a ${SasOutLog}
    grep -i " ERROR " tmp.out |& tee -a ${SasOutLog}
    /bin/rm -f tmp.out
endif

if (${viableM1data} > 0) then
    if (-e tmp.out) /bin/rm -f tmp.out
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viableM1data} )
	#
	# SRSPEC (events with all valid patterns, circular aperture):
	set tmpSPEC0 = ${M1SPEClist[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${M1BGSPEClist[$tmpCounter]}.gz"
	    set tmpRMF   = "${M1RMF_list[$tmpCounter]}.gz"
	    set tmpARF   = "${M1ARF_list[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-13 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-13 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	#
	# SRSP_0 (only pattern==0 events, circular aperture):
	set tmpSPEC0 = ${M1SRSP_0list[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${M1BGSP_0list[$tmpCounter]}.gz"
	    set tmpRMF   = "${M1RMF0list[$tmpCounter]}.gz"
	    set tmpARF   = "${M1ARF0list[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-13 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-13 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	#
	# SRSP_A (events with all valid patterns, annular aperture):
	set tmpSPEC0 = ${M1SRSP_Alist[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${M1BGSPEClist[$tmpCounter]}.gz"
	    set tmpRMF   = "${M1RMF_list[$tmpCounter]}.gz"
	    set tmpARF   = "${M1ARFAlist[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-13 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-13 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	@ tmpCounter++
    end
    echo "Output from running GRPPHA on M1 spectra:" >>! ${SasOutLog}
    cat tmp.out >>! ${SasOutLog}
    # echo "If any errors were encountered when running GRPPHA on the M1 spectra," |& tee -a ${SasOutLog}
    # echo "   these appear on the following lines:" |& tee -a ${SasOutLog}
    echo "       GRPPHA warnings and errors for M1 spectra:" |& tee -a ${SasOutLog}
    grep -i " ERROR " tmp.out |& tee -a ${SasOutLog}
    /bin/rm -f tmp.out
endif

if (${viableM2data} > 0) then
    if (-e tmp.out) /bin/rm -f tmp.out
    set tmpCounter = 1
    while ( ${tmpCounter} <= ${viableM2data} )
	#
	# SRSPEC (events with all valid patterns, circular aperture):
	set tmpSPEC0 = ${M2SPEClist[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${M2BGSPEClist[$tmpCounter]}.gz"
	    set tmpRMF   = "${M2RMF_list[$tmpCounter]}.gz"
	    set tmpARF   = "${M2ARF_list[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-13 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-13 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	#
	# SRSP_0 (only pattern==0 events, circular aperture):
	set tmpSPEC0 = ${M2SRSP_0list[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${M2BGSP_0list[$tmpCounter]}.gz"
	    set tmpRMF   = "${M2RMF0list[$tmpCounter]}.gz"
	    set tmpARF   = "${M2ARF0list[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-13 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-13 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	#
	# SRSP_A (events with all valid patterns, annular aperture):
	set tmpSPEC0 = ${M2SRSP_Alist[$tmpCounter]}
	if (-e $tmpSPEC0) then
	    set tmpBACK  = "${M2BGSPEClist[$tmpCounter]}.gz"
	    set tmpRMF   = "${M2RMF_list[$tmpCounter]}.gz"
	    set tmpARF   = "${M2ARFAlist[$tmpCounter]}.gz"
	    set tmpSPEC1 = `echo ${tmpSPEC0} | sed -e "s/.F/_grp00.F/g" -e "s/.FTZ/.FIT/g"`
	    set tmpSPEC2 = `echo ${tmpSPEC1} | sed -e "s/grp00/grp25/g"`
	    fkeypar ${tmpSPEC0} BACKSCAL
	    set tempBackScal = `pget fkeypar value`
	    set BackScalTest = `calc ${tempBackScal} - ${minSrcArea} | awk '{print substr($0,1,1)}'`
	    if (-e $tmpSPEC1) /bin/rm $tmpSPEC1
	    # Ideally, here I should only rm $tmpSPEC1 if it is older than $tmpSPEC0...
	    if ("${BackScalTest}" != "-") then
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & chkey RESPFILE $tmpRMF & chkey ANCRFILE $tmpARF & reset quality & bad 0-13 & exit" >>&! tmp.out
	    else
		grppha $tmpSPEC0 $tmpSPEC1 "chkey BACKFILE $tmpBACK & reset quality & bad 0-13 & exit" >>&! tmp.out
	    endif
	    if (-e $tmpSPEC2) /bin/rm $tmpSPEC2
	    # Ideally, here I should only rm $tmpSPEC2 if it is older than $tmpSPEC1...
	    grppha $tmpSPEC1 $tmpSPEC2 "group min 25 & exit" >>&! tmp.out
	endif
	@ tmpCounter++
    end
    echo "Output from running GRPPHA on M2 spectra:" >>! ${SasOutLog}
    cat tmp.out >>! ${SasOutLog}
    # echo "If any errors were encountered when running GRPPHA on the M2 spectra," |& tee -a ${SasOutLog}
    # echo "   these appear on the following lines:" |& tee -a ${SasOutLog}
    echo "       GRPPHA warnings and errors for M2 spectra:" |& tee -a ${SasOutLog}
    grep -i " ERROR " tmp.out |& tee -a ${SasOutLog}
    /bin/rm -f tmp.out
endif

if (${?tmpSPEC0})	unset tmpSPEC0
if (${?tmpBACK})	unset tmpBACK
if (${?tmpRMF}) 	unset tmpRMF
if (${?tmpARF}) 	unset tmpARF
if (${?tmpSPEC1})	unset tmpSPEC1
if (${?tmpSPEC2})	unset tmpSPEC2
if (${?tmpCounter})	unset tmpCounter
if (${?tempBackScal})	unset tempBackScal
if (${?BackScalTest})	unset BackScalTest


echo " " |& tee -a ${SasOutLog}
#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (13) <><>" |& tee -a ${SasOutLog}
#echo "<><>             rearranging files & reclaiming disk space <><>" |& tee -a ${SasOutLog}
echo "<><><><> ...tidyng up, compressing files, reclaiming disk space... <><><><>" |& tee -a ${SasOutLog}
#############
# (13) copy/move all source-specific files needed by XSPEC or for
# diagnostic purposes to work directory, then go there...
#
# echo " "
# echo "Lastly, doing some clean-up, file compression and set up for later XSPEC use."
# echo " "

gzip -9vf *FIT >>&! ${SasOutLog}
gzip -9vf *ps >>&! ${SasOutLog}
/bin/rm -rf ../odf

# To keep clutter down, I'll create subdirectories for diagnostic figures,
# scripts used (and their output) and region files.
#
mkdir ${XSPEC_WORKDIR}/diagnostics
mkdir ${XSPEC_WORKDIR}/scripts
mkdir ${XSPEC_WORKDIR}/regions
/bin/cp -fpRP P${OBSID_XMM}*-hist.* ${XSPEC_WORKDIR}/diagnostics
/bin/cp -fpRP P${OBSID_XMM}*IM_*${SRCNUM_XMM}.* ${XSPEC_WORKDIR}/diagnostics
/bin/cp -fpRP P${OBSID_XMM}*${SRCNUM_XMM}_pat.* ${XSPEC_WORKDIR}/diagnostics
/bin/cp -fpRP ${ScriptPath_XMM}*.csh ${XSPEC_WORKDIR}/scripts
/bin/cp -fpRP ${SasOutLog} ${XSPEC_WORKDIR}/diagnostics
/bin/cp -fpRP P${OBSID_XMM}*${SRCNUM_XMM}.reg ${XSPEC_WORKDIR}/regions

# I might only copy annular or pattern==0 source extractions to the work directory
#   if the count rates are high enough to make these useful...  (I should do this
#   based upon the count rate and observing setup, but for now it's just a hard-
#   coded flag...)
#
/bin/cp -fpRP P${OBSID_XMM}*SRSPEC0${SRCNUM_XMM}_grp*.F* ${XSPEC_WORKDIR}  # standard src spec
/bin/cp -fpRP P${OBSID_XMM}*BGSPEC0${SRCNUM_XMM}*.${specFITSsuffix}* ${XSPEC_WORKDIR}	# standard bgd spec
/bin/cp -fpRP P${OBSID_XMM}*SRRMF_0${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}	# standard RMFs
/bin/cp -fpRP P${OBSID_XMM}*SRARF_0${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}	# standard ARFs
#
if (${possiblePileUp} > 0) then
    /bin/cp -fpRP P${OBSID_XMM}*SRSP_00${SRCNUM_XMM}_grp*.F* $XSPEC_WORKDIR # grade 0 circular
    /bin/cp -fpRP P${OBSID_XMM}*SRSP_A0${SRCNUM_XMM}_grp*.F* $XSPEC_WORKDIR #all grade annular
    /bin/cp -fpRP P${OBSID_XMM}*BGSP_00${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}  # grad0 0 background
    /bin/cp -fpRP P${OBSID_XMM}*SRARF00${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}  # grade 0 arf
    /bin/cp -fpRP P${OBSID_XMM}*SRARFA0${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}  # annular arf 
    /bin/cp -fpRP P${OBSID_XMM}*SRRMF00${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}  # grade 0 rmf
endif
#
# More compact, non-discriminating way to copy all spectral products:
#
# /bin/cp -fpRP P${OBSID_XMM}*SRSP*0${SRCNUM_XMM}_grp*.F* ${XSPEC_WORKDIR}  # all source spec
# /bin/cp -fpRP P${OBSID_XMM}*BGSP*0${SRCNUM_XMM}.F* ${XSPEC_WORKDIR}  # all background spec
# /bin/cp -fpRP P${OBSID_XMM}*SR??F?0${SRCNUM_XMM}.F* ${XSPEC_WORKDIR} # all response matrices

# This will be useful when creating XCM files...
#
set tmpPNspec0List = `/bin/ls P${OBSID_XMM}PN*SRSP_00${SRCNUM_XMM}_grp25.F* \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ${XSPEC_WORKDIR}/diagnostics/${SasOutLog}`

# Lastly, change from SAS work directory to XSPEC analysis directory
#
cd ${XSPEC_WORKDIR}


# echo " " |& tee -a ./diagnostics/${SasOutLog}
# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (14) <><>" |& tee -a ./diagnostics/${SasOutLog}
# echo "<><>             measuring Galactic NH                     <><>" |& tee -a ./diagnostics/${SasOutLog}
echo "<><><><> ...fetching Galactic NH column density... <><><><>" |& tee -a ${SasOutLog}
echo " " |& tee -a ./diagnostics/${SasOutLog}
#############
# (14) get the Galactic NH column
#
nh equinox=2000 ra=${SDSS_RA} dec=${SDSS_DEC} tchat=5 lchat=10 usemap=2 |& tee -a ./diagnostics/${SasOutLog}
setenv GalNH `pget nh avwnh`


echo " " |& tee -a ./diagnostics/${SasOutLog}
# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (15) <><>" |& tee -a ./diagnostics/${SasOutLog}
# echo "<><>             this section does nothing                 <><>" |& tee -a ./diagnostics/${SasOutLog}
#############
# (15) Set symbolic links to point to default set of spectra
#
# For now, I'll assume I want grouped spectra and not ungrouped ones.
# Need to implement a test for this...
#
# ln -s ${PN_SP1}.gz spec_PN_noGrp.fit
# ln -s ${M1_SP1}.gz spec_M1_noGrp.fit
# ln -s ${M2_SP1}.gz spec_M2_noGrp.fit
# ln -s ${PN_SP2}.gz spec_PN_grp25.fit
# ln -s ${M1_SP2}.gz spec_M1_grp25.fit
# ln -s ${M2_SP2}.gz spec_M2_grp25.fit

# echo "<><> Debug flag: now entering xmmSpecPrep.csh section (16) <><>" |& tee -a ./diagnostics/${SasOutLog}
# echo "<><>             writing XCM setup files for XSPEC         <><>" |& tee -a ./diagnostics/${SasOutLog}
echo "<><><><> ...setting up XSPEC command files... <><><><>" |& tee -a ${SasOutLog}
#############
# (16) Write file with inputs for XSPEC processing script(s)
# ...perhaps this would be better done as part of XSPEC script...
# ...or perhaps I should set up the XSPEC scripts now...
#
# echo $GalNH ... > xspec_in.txt

if (-e xspec_setup.xcm) /bin/rm -f xspec_setup.xcm
if (-e files_grp00.xcm) /bin/rm -f files_grp00.xcm
if (-e files_grp25.xcm) /bin/rm -f files_grp25.xcm

echo "setplot energy" >>! xspec_setup.xcm
echo "cosmo 70.5,,0.726" >>! xspec_setup.xcm
echo "method leven 200 0.0005" >>! xspec_setup.xcm
echo "weight churazov" >>! xspec_setup.xcm
echo "abund angr" >>! xspec_setup.xcm
echo "setplot command font roman" >>! xspec_setup.xcm
echo "setplot command time off" >>! xspec_setup.xcm
echo setplot command la ot \"${IAUNAME_XMM} \(${OBSID_XMM} src\#${SRCNUM_XMM}\; DETID\=${DETID_XMM}\; ${COMMONNAME}\)\" >>! xspec_setup.xcm

echo "@xspec_setup.xcm" >>! files_grp00.xcm
echo "@xspec_setup.xcm" >>! files_grp25.xcm

set tmpPNspecList = `/bin/ls P${OBSID_XMM}PN*SRSPEC0${SRCNUM_XMM}_grp25.F* \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ./diagnostics/${SasOutLog}`
set tmpM1specList = `/bin/ls P${OBSID_XMM}M1*SRSPEC0${SRCNUM_XMM}_grp25.F* \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ./diagnostics/${SasOutLog}`
set tmpM2specList = `/bin/ls P${OBSID_XMM}M2*SRSPEC0${SRCNUM_XMM}_grp25.F* \
    |& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g" |& tee -a ./diagnostics/${SasOutLog}`
# Defined earlier, before copying files and changing directories...
# set tmpPNspec0List = `/bin/ls P${OBSID_XMM}PN*SRSP_00${SRCNUM_XMM}_grp25.F*`

set tmpCounter = 1
foreach ii ( ${tmpPNspecList} ${tmpM1specList} ${tmpM2specList} )
    fkeypar "${ii}[1]" EXPOSURE
    set tempExpTime = `pget fkeypar value`
    set expTimeTest = `calc ${tempExpTime} - ${minExpTime} | awk '{print substr($0,1,1)}'`
    if ("${expTimeTest}" != "-") then
	set tmpSPEC = `echo ${ii} | sed -e "s/grp25/grp00/g"`
	echo "data ${tmpCounter}:${tmpCounter} ${ii}" >>! files_grp25.xcm
	echo "data ${tmpCounter}:${tmpCounter} ${tmpSPEC}" >>! files_grp00.xcm
	@ tmpCounter++
    endif
end
@ tmpCounter--
echo "ignore bad" >>! files_grp00.xcm
echo "ignore bad" >>! files_grp25.xcm
echo "ignore 1-${tmpCounter}:**-0.2" >>! files_grp00.xcm
echo "ignore 1-${tmpCounter}:**-0.2" >>! files_grp25.xcm
echo "statistic cstat" >>! files_grp00.xcm
echo "statistic chi" >>! files_grp25.xcm
echo "setplot list" >>! files_grp00.xcm
echo "setplot list" >>! files_grp25.xcm

if ("${tmpPNspec0List}"x != x) then
    if (-e files_grp00_PNlowEnoise.xcm) /bin/rm -f files_grp00_PNlowEnoise.xcm
    if (-e files_grp25_PNlowEnoise.xcm) /bin/rm -f files_grp25_PNlowEnoise.xcm
    echo "@xspec_setup.xcm" >>! files_grp00_PNlowEnoise.xcm
    echo "@xspec_setup.xcm" >>! files_grp25_PNlowEnoise.xcm

    set tmpCounter = 1
    set tmpPNCounter = 1
    set tmpPN0Counter = 1
    set tmpMOSCounter = 1

    foreach ii ( ${tmpPNspecList} )
	fkeypar "${ii}[1]" EXPOSURE
	set tempExpTime = `pget fkeypar value`
	set expTimeTest = `calc ${tempExpTime} - ${minExpTime} | awk '{print substr($0,1,1)}'`
	if ("${expTimeTest}" != "-") then
	    set tmpSPEC = `echo ${ii} | sed -e "s/grp25/grp00/g"`
	    echo "data ${tmpCounter}:${tmpCounter} ${ii}" >>! files_grp25_PNlowEnoise.xcm
	    echo "data ${tmpCounter}:${tmpCounter} ${tmpSPEC}" >>! files_grp00_PNlowEnoise.xcm
	    @ tmpCounter++
	    @ tmpPNCounter++
	endif
    end
    @ tmpCounter--
    @ tmpPNCounter--

    foreach ii ( ${tmpPNspec0List} )
	if (-e ${ii}) then
	    # Note that *PN*SRSP_0* files may not be present...
	    fkeypar "${ii}[1]" EXPOSURE
	    set tempExpTime = `pget fkeypar value`
	else
	    # ...in which case we'll insert them anyway...
	    set tempExpTime = `calc ${minExpTime} + 1`
	endif
	set expTimeTest = `calc ${tempExpTime} - ${minExpTime} | awk '{print substr($0,1,1)}'`
	if ("${expTimeTest}" != "-") then
	    set tmpSPEC = `echo ${ii} | sed -e "s/grp25/grp00/g"`
	    echo "data ${tmpCounter}:${tmpCounter} ${ii}" >>! files_grp25_PNlowEnoise.xcm
	    echo "data ${tmpCounter}:${tmpCounter} ${tmpSPEC}" >>! files_grp00_PNlowEnoise.xcm
	    @ tmpCounter++
	    @ tmpPN0Counter++
	endif
    end
    @ tmpCounter--
    @ tmpPN0Counter--
    
    foreach ii ( ${tmpM1specList} ${tmpM2specList} )
	fkeypar "${ii}[1]" EXPOSURE
	set tempExpTime = `pget fkeypar value`
	set expTimeTest = `calc ${tempExpTime} - ${minExpTime} | awk '{print substr($0,1,1)}'`
	if ("${expTimeTest}" != "-") then
	    set tmpSPEC = `echo ${ii} | sed -e "s/grp25/grp00/g"`
	    echo "data ${tmpCounter}:${tmpCounter} ${ii}" >>! files_grp25_PNlowEnoise.xcm
	    echo "data ${tmpCounter}:${tmpCounter} ${tmpSPEC}" >>! files_grp00_PNlowEnoise.xcm
	    @ tmpCounter++
	    @ tmpMOSCounter++
	endif
    end
    @ tmpCounter--
    @ tmpMOSCounter--
    
    echo "ignore bad" >>! files_grp00_PNlowEnoise.xcm
    echo "ignore bad" >>! files_grp25_PNlowEnoise.xcm
    echo "ignore 1-${tmpCounter}:**-0.2" >>! files_grp00_PNlowEnoise.xcm
    echo "ignore 1-${tmpCounter}:**-0.2" >>! files_grp25_PNlowEnoise.xcm
    if ( ${tmpPNCounter} > 0 ) then
	echo "ignore 1-${tmpPNCounter}:**-0.5" >>! files_grp00_PNlowEnoise.xcm
	echo "ignore 1-${tmpPNCounter}:**-0.5" >>! files_grp25_PNlowEnoise.xcm
    endif
    if ( ${tmpPN0Counter} > 0 ) then
	set tmpLowerLim = `calc ${tmpPNCounter} + 1`
	set tmpUpperLim = `calc ${tmpPNCounter} + ${tmpPN0Counter}`
	echo "ignore ${tmpLowerLim}-${tmpUpperLim}:0.5-**" >>! files_grp00_PNlowEnoise.xcm
	echo "ignore ${tmpLowerLim}-${tmpUpperLim}:0.5-**" >>! files_grp25_PNlowEnoise.xcm
    endif
    unset tmpPNCounter
    unset tmpPN0Counter
    unset tmpMOSCounter
    echo "statistic cstat" >>! files_grp00_PNlowEnoise.xcm
    echo "statistic chi" >>! files_grp25_PNlowEnoise.xcm
    echo "setplot list" >>! files_grp00_PNlowEnoise.xcm
    echo "setplot list" >>! files_grp25_PNlowEnoise.xcm
endif

unset tmpCounter
unset ii
unset tmpSPEC

# Separate scripts for different stages of spectral analysis?
# Near-identical/redundant scripts using ungrouped data and Cash statistics?

# Either generate scripts here, or generate a file with input parameters that
# will be needed by a later script?  Could set up "save files" XCMs.
# If I'll run XSPEC through IDL, taking in fit results and using these as the
# next scripts are written, IDL will need the names of the various save files
# XCMs, the number of exposures for each instrument, the galactic NH column
# estimate, ...

echo " " >>! ./diagnostics/${SasOutLog}
echo " " |& tee -a ./diagnostics/${SasOutLog}
echo "Finished pre-processing $IAUNAME_XMM" |& tee -a ./diagnostics/${SasOutLog}
echo "                        (DETID ${DETID_XMM}; OBSID ${OBSID_XMM}, SRCNUM ${SRCNUM_XMM} ${COMMONNAME})" |& tee -a ./diagnostics/${SasOutLog}
echo " " |& tee -a ./diagnostics/${SasOutLog}
echo "Processing had begun at ${SASstartTime}" |& tee -a ./diagnostics/${SasOutLog}
echo "    It was completed at `date`" |& tee -a ./diagnostics/${SasOutLog}
echo " " |& tee -a ./diagnostics/${SasOutLog}
echo " " >>! ./diagnostics/${SasOutLog}

# Get out of XSPEC analysis directory:
popd
#
# ...also get out of XMM data directory if we didn't start there:
if (${?moveToDataPath}) then
    popd
    unsetenv moveToDataPath
endif

# This file captures a copy of what appears in the terminal window, so
# make sure there is no output to the terminal after it is moved...
#
if (-e ${DataPath_XMM}${OBSID_XMM}_${SRCNUM_XMM}_summary.log) then
    enscript -h3rc -o ${DataPath_XMM}${OBSID_XMM}_${SRCNUM_XMM}_summary.ps \
	-f Times-Roman7 -M A4 ${DataPath_XMM}${OBSID_XMM}_${SRCNUM_XMM}_summary.log
    /bin/mv -f ${DataPath_XMM}${OBSID_XMM}_${SRCNUM_XMM}_summary.* ${XSPEC_WORKDIR}/diagnostics/
endif

#echo "<><> Debug flag: now entering xmmSpecPrep.csh section (0) <><>"
#echo "<><>             resetting left-over parameters           <><>"
#############
# (0) Make sure variables are reset before later runs...
#
if (${?viablePNdata}) 	unsetenv viablePNdata
if (${?viableM1data}) 	unsetenv viableM1data
if (${?viableM2data}) 	unsetenv viableM2data
if (${?DETID_XMM}) 	unsetenv DETID_XMM
if (${?OBSID_XMM}) 	unsetenv OBSID_XMM
if (${?IAUNAME_XMM}) 	unsetenv IAUNAME_XMM
if (${?SRCNUM_XMM}) 	unsetenv SRCNUM_XMM
if (${?SDSS_RA}) 	unsetenv SDSS_RA
if (${?SDSS_DEC}) 	unsetenv SDSS_DEC
if (${?COMMONNAME}) 	unsetenv COMMONNAME
if (${?ScriptPath_XMM}) 	unsetenv ScriptPath_XMM
if (${?DataPath_XMM}) 	unsetenv DataPath_XMM
if (${?AnalysisPath}) 	unsetenv AnalysisPath
if (${?SasOutLog}) 	unsetenv SasOutLog
if (${?UsePPSspectra}) 	unsetenv UsePPSspectra
if (${?UsePPSimages}) 	unsetenv UsePPSimages
if (${?IAUNAME_TRUN}) 	unsetenv IAUNAME_TRUN
if (${?NAME_TRUN}) 	unsetenv NAME_TRUN
if (${?XSPEC_WORKDIR}) 	unsetenv XSPEC_WORKDIR
if (${?SAS_ODF}) 	unsetenv SAS_ODF
if (${?SAS_CCF}) 	unsetenv SAS_CCF
if (${?specFITSsuffix}) 	unsetenv specFITSsuffix
if (${?imageFITSsuffix}) 	unsetenv imageFITSsuffix
if (${?DummyRegFilt}) 	unsetenv DummyRegFilt
if (${?SrcCirFileOut}) 	unsetenv SrcCirFileOut
if (${?SrcAnnFileOut}) 	unsetenv SrcAnnFileOut
if (${?BgdAnnFileOut}) 	unsetenv BgdAnnFileOut
if (${?BgdObjFileOut}) 	unsetenv BgdObjFileOut
if (${?wideFieldJpg}) 	unsetenv wideFieldJpg
if (${?wideFieldEps}) 	unsetenv wideFieldEps
if (${?zoomFieldJpg}) 	unsetenv zoomFieldJpg
if (${?zoomFieldEps}) 	unsetenv zoomFieldEps
if (${?GalNH})  	unsetenv GalNH
if (${?SASstartTime})	unset SASstartTime
if (${?BgdAnnRegStr}) 	unsetenv BgdAnnRegStr
if (${?BgdObjRegStr}) 	unsetenv BgdObjRegStr
if (${?SrcAnnRegStr}) 	unsetenv SrcAnnRegStr
if (${?SrcCirRegStr}) 	unsetenv SrcCirRegStr
if (${?PNARF0list}) 	unset PNARF0list
if (${?PNARFAlist}) 	unset PNARFAlist
if (${?PNARF_list}) 	unset PNARF_list
if (${?PNBGSPEClist}) 	unset PNBGSPEClist
if (${?PNBGSP_0list}) 	unset PNBGSP_0list
if (${?PNRMF0list}) 	unset PNRMF0list
if (${?PNRMF_list}) 	unset PNRMF_list
if (${?PNSPEClist}) 	unset PNSPEClist
if (${?PNSRSP_0list}) 	unset PNSRSP_0list
if (${?PNSRSP_Alist}) 	unset PNSRSP_Alist
if (${?PN_BgdAnnRegFile}) 	unset PN_BgdAnnRegFile
if (${?PN_BgdAnnRegFilt}) 	unset PN_BgdAnnRegFilt
if (${?PN_BgdObjRegFile}) 	unset PN_BgdObjRegFile
if (${?PN_BgdObjRegFilt}) 	unset PN_BgdObjRegFilt
if (${?PN_EVTlist}) 	unset PN_EVTlist
if (${?PN_EXPID}) 	unset PN_EXPID
if (${?PN_FILTER}) 	unset PN_FILTER
if (${?PN_FlareGTI}) 	unset PN_FlareGTI
if (${?PN_SUBMODE}) 	unset PN_SUBMODE
if (${?PN_SrcAnnRegFile}) 	unset PN_SrcAnnRegFile
if (${?PN_SrcAnnRegFilt}) 	unset PN_SrcAnnRegFilt
if (${?PN_SrcCirRegFile}) 	unset PN_SrcCirRegFile
if (${?PN_SrcCirRegFilt}) 	unset PN_SrcCirRegFilt
if (${?PNimage1list}) 	unset PNimage1list
if (${?PNimage8list}) 	unset PNimage8list
if (${?M1ARF0list}) 	unset M1ARF0list
if (${?M1ARFAlist}) 	unset M1ARFAlist
if (${?M1ARF_list}) 	unset M1ARF_list
if (${?M1BGSPEClist}) 	unset M1BGSPEClist
if (${?M1BGSP_0list}) 	unset M1BGSP_0list
if (${?M1RMF0list}) 	unset M1RMF0list
if (${?M1RMF_list}) 	unset M1RMF_list
if (${?M1SPEClist}) 	unset M1SPEClist
if (${?M1SRSP_0list}) 	unset M1SRSP_0list
if (${?M1SRSP_Alist}) 	unset M1SRSP_Alist
if (${?M1_BgdAnnRegFile}) 	unset M1_BgdAnnRegFile
if (${?M1_BgdAnnRegFilt}) 	unset M1_BgdAnnRegFilt
if (${?M1_BgdObjRegFile}) 	unset M1_BgdObjRegFile
if (${?M1_BgdObjRegFilt}) 	unset M1_BgdObjRegFilt
if (${?M1_EVTlist}) 	unset M1_EVTlist
if (${?M1_EXPID}) 	unset M1_EXPID
if (${?M1_FILTER}) 	unset M1_FILTER
if (${?M1_FlareGTI}) 	unset M1_FlareGTI
if (${?M1_SUBMODE}) 	unset M1_SUBMODE
if (${?M1_SrcAnnRegFile}) 	unset M1_SrcAnnRegFile
if (${?M1_SrcAnnRegFilt}) 	unset M1_SrcAnnRegFilt
if (${?M1_SrcCirRegFile}) 	unset M1_SrcCirRegFile
if (${?M1_SrcCirRegFilt}) 	unset M1_SrcCirRegFilt
if (${?M1image8list}) 	unset M1image8list
if (${?M2ARF0list}) 	unset M2ARF0list
if (${?M2ARFAlist}) 	unset M2ARFAlist
if (${?M2ARF_list}) 	unset M2ARF_list
if (${?M2BGSPEClist}) 	unset M2BGSPEClist
if (${?M2BGSP_0list}) 	unset M2BGSP_0list
if (${?M2RMF0list}) 	unset M2RMF0list
if (${?M2RMF_list}) 	unset M2RMF_list
if (${?M2SPEClist}) 	unset M2SPEClist
if (${?M2SRSP_0list}) 	unset M2SRSP_0list
if (${?M2SRSP_Alist}) 	unset M2SRSP_Alist
if (${?M2_BgdAnnRegFile}) 	unset M2_BgdAnnRegFile
if (${?M2_BgdAnnRegFilt}) 	unset M2_BgdAnnRegFilt
if (${?M2_BgdObjRegFile}) 	unset M2_BgdObjRegFile
if (${?M2_BgdObjRegFilt}) 	unset M2_BgdObjRegFilt
if (${?M2_EVTlist}) 	unset M2_EVTlist
if (${?M2_EXPID}) 	unset M2_EXPID
if (${?M2_FILTER}) 	unset M2_FILTER
if (${?M2_FlareGTI}) 	unset M2_FlareGTI
if (${?M2_SUBMODE}) 	unset M2_SUBMODE
if (${?M2_SrcAnnRegFile}) 	unset M2_SrcAnnRegFile
if (${?M2_SrcAnnRegFilt}) 	unset M2_SrcAnnRegFilt
if (${?M2_SrcCirRegFile}) 	unset M2_SrcCirRegFile
if (${?M2_SrcCirRegFilt}) 	unset M2_SrcCirRegFilt
if (${?M2image8list}) 	unset M2image8list
if (${?SrcEvLi}) 	unset SrcEvLi
if (${?extractM1SPEC}) 	unset extractM1SPEC
if (${?extractM1image8}) 	unset extractM1image8
if (${?extractM2SPEC}) 	unset extractM2SPEC
if (${?extractM2image8}) 	unset extractM2image8
if (${?extractPNSPEC}) 	unset extractPNSPEC
if (${?extractPNimage1}) 	unset extractPNimage1
if (${?extractPNimage8}) 	unset extractPNimage8
if (${?tmpSPECname}) 	unset tmpSPECname
if (${?tempRegList}) 	unset tempRegList
if (${?ds9scaleMode})	unsetenv ds9scaleMode
